@@ -3,8 +3,11 @@ set -e
 set -x
 shopt -s extglob
 
-#install anything missing from the container
-./contrib/ci/generate_dependencies.py | xargs pacman -S --noconfirm --needed
+#clone test firmware
+if [ "$CI_NETWORK" = "true" ]; then
+	./contrib/ci/get_test_firmware.sh
+	export G_TEST_SRCDIR=`pwd`/fwupd-test-firmware/installed-tests
+fi
 
 # prepare the build tree
 rm -rf build
@@ -16,8 +19,8 @@ popd
 chown nobody . -R
 
 # install and run TPM simulator necessary for plugins/uefi/uefi-self-test
-pacman -S --noconfirm ibm-sw-tpm2 tpm2-tools
-tpm_server &
+pacman -Syu --noconfirm swtpm tpm2-tools
+swtpm socket --tpm2 --server port=2321 --ctrl type=tcp,port=2322 --flags not-need-init --tpmstate "dir=$PWD" &
 trap "kill $!" EXIT
 # extend a PCR0 value for test suite
 sleep 2
@@ -29,6 +32,11 @@ export TPM_SERVER_RUNNING=1
 sudo -E -u nobody PKGEXT='.pkg.tar' makepkg -e --noconfirm
 pacman -U --noconfirm *.pkg.*
 
+#run the CI tests for Qt5
+pacman -Syu --noconfirm qt5-base
+meson qt5-thread-test ../contrib/ci/qt5-thread-test
+ninja -C qt5-thread-test test
+
 # move the package to working dir
 mv *.pkg.* ../dist
 
