@@ -15,6 +15,7 @@ meson .. \
     --libexecdir=$target \
     --bindir=$target \
     -Dbuild=standalone \
+    -Dpolkit=false \
     -Dplugin_coreboot=false \
     -Dplugin_flashrom=false \
     -Dplugin_uefi=false \
@@ -22,6 +23,7 @@ meson .. \
     -Dplugin_altos=false \
     -Dplugin_dell=false \
     -Dplugin_nvme=false \
+    -Dplugin_platform_integrity=false \
     -Dtpm=false \
     -Dsystemd=false \
     -Dplugin_emmc=false \
@@ -30,6 +32,7 @@ meson .. \
     -Dplugin_thunderbolt=false \
     -Dplugin_synaptics=false \
     -Dman=false \
+    -Dsoup_session_compat=false \
     -Dgcab:introspection=false \
     -Dgcab:docs=false \
     -Dgcab:nls=false \
@@ -40,6 +43,10 @@ meson .. \
     -Dlibjcat:man=false \
     -Dlibjcat:gpg=false \
     -Dlibjcat:introspection=false \
+    -Dgusb:tests=false \
+    -Dgusb:docs=false \
+    -Dgusb:introspection=false \
+    -Dgusb:vapi=false \
     -Dgudev=false $@
 VERSION=$(meson introspect . --projectinfo | jq -r .version)
 ninja -v
