@@ -2,6 +2,9 @@
 set -e
 set -x
 
+# remove when tpm2-tss is fixed
+mkdir -p /usr/include/tss
+
 #although it's debian, we don't build packages
 if [ "$OS" = "debian-s390x" ]; then
 	./contrib/ci/debian_s390x.sh
@@ -26,17 +29,21 @@ sed s/quilt/native/ debian/source/format -i
 #check if we have all deps available
 #if some are missing, we're going to use subproject instead and
 #packaging CI will fail
-./contrib/ci/generate_dependencies.py  | xargs apt install -y || true
 if ! dpkg-checkbuilddeps; then
 	./contrib/ci/ubuntu.sh
 	exit 0
 fi
 
+#clone test firmware
+if [ "$CI_NETWORK" = "true" ]; then
+	./contrib/ci/get_test_firmware.sh
+	export G_TEST_SRCDIR=`pwd`/fwupd-test-firmware/installed-tests
+fi
+
 #disable unit tests if fwupd is already installed (may cause problems)
 if [ -x /usr/lib/fwupd/fwupd ]; then
 	export DEB_BUILD_OPTIONS=nocheck
 fi
-
 #build the package
 EDITOR=/bin/true dch --create --package fwupd -v $VERSION "CI Build"
 debuild --no-lintian --preserve-envvar CI --preserve-envvar CC
