@@ -2,9 +2,6 @@
 set -e
 set -x
 
-#get any missing deps from the container
-./contrib/ci/generate_dependencies.py | xargs dnf install -y
-
 #generate a tarball
 git config tar.tar.xz.command "xz -c"
 mkdir -p build && pushd build
@@ -57,7 +54,7 @@ mkdir -p dist
 cp $HOME/rpmbuild/RPMS/*/*.rpm dist
 
 if [ "$CI" = "true" ]; then
-	sed "s,^BlacklistPlugins=test;invalid,BlacklistPlugins=," -i /etc/fwupd/daemon.conf
+	sed "s,^DisabledPlugins=test;invalid,DisabledPlugins=," -i /etc/fwupd/daemon.conf
 
 	# set up enough PolicyKit and D-Bus to run the daemon
 	mkdir -p /run/dbus
