@@ -43,7 +43,7 @@ rc=$?; if [[ $rc != 0 ]]; then error $rc; fi
 
 # ---
 echo "Installing test firmware..."
-fwupdmgr install ${dirname}/fakedevice124.cab
+fwupdmgr update $device -y
 rc=$?; if [[ $rc != 0 ]]; then error $rc; fi
 
 # ---
@@ -56,9 +56,14 @@ echo "Testing the verification of firmware (again)..."
 fwupdmgr verify $device
 rc=$?; if [[ $rc != 0 ]]; then error $rc; fi
 
+if [ -z "$CI_NETWORK" ]; then
+        echo "Skipping remaining tests due to CI_NETWORK not being set"
+        exit 0
+fi
+
 # ---
 echo "Downgrading to older release (requires network access)"
-fwupdmgr downgrade $device
+fwupdmgr downgrade $device -y
 rc=$?; if [[ $rc != 0 ]]; then error $rc; fi
 
 # ---
@@ -68,7 +73,7 @@ rc=$?; if [[ $rc != 2 ]]; then error $rc; fi
 
 # ---
 echo "Updating all devices to latest release (requires network access)"
-fwupdmgr --no-unreported-check --no-metadata-check --no-reboot-check update
+fwupdmgr --no-unreported-check --no-metadata-check --no-reboot-check update -y
 rc=$?; if [[ $rc != 0 ]]; then error $rc; fi
 
 # ---
