@@ -6,7 +6,7 @@
 
 #include "config.h"
 
-#include <libsoup/soup.h>
+#include <curl/curl.h>
 #include <jcat.h>
 
 #include "fwupd-deprecated.h"
@@ -32,6 +32,7 @@ typedef struct {
 	gchar			*id;
 	gchar			*firmware_base_uri;
 	gchar			*report_uri;
+	gchar			*security_report_uri;
 	gchar			*metadata_uri;
 	gchar			*metadata_uri_sig;
 	gchar			*username;
@@ -50,6 +51,7 @@ typedef struct {
 	gchar			**order_before;
 	gchar			*remotes_dir;
 	gboolean		 automatic_reports;
+	gboolean		 automatic_security_reports;
 } FwupdRemotePrivate;
 
 enum {
@@ -58,12 +60,19 @@ enum {
 	PROP_ENABLED,
 	PROP_APPROVAL_REQUIRED,
 	PROP_AUTOMATIC_REPORTS,
+	PROP_AUTOMATIC_SECURITY_REPORTS,
 	PROP_LAST
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (FwupdRemote, fwupd_remote, G_TYPE_OBJECT)
 #define GET_PRIVATE(o) (fwupd_remote_get_instance_private (o))
 
+#ifdef HAVE_LIBCURL_7_62_0
+typedef gchar curlptr;
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(curlptr, curl_free)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(CURLU, curl_url_cleanup)
+#endif
+
 static void
 fwupd_remote_set_username (FwupdRemote *self, const gchar *username)
 {
@@ -122,7 +131,16 @@ fwupd_remote_set_kind (FwupdRemote *self, FwupdRemoteKind kind)
 	priv->kind = kind;
 }
 
-static void
+/**
+ * fwupd_remote_set_keyring_kind:
+ * @self: A #FwupdRemote
+ * @keyring_kind: #FwupdKeyringKind e.g. #FWUPD_KEYRING_KIND_PKCS7
+ *
+ * Sets the keyring kind
+ *
+ * Since: 1.5.3
+ **/
+void
 fwupd_remote_set_keyring_kind (FwupdRemote *self, FwupdKeyringKind keyring_kind)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE (self);
@@ -159,60 +177,53 @@ fwupd_remote_get_suffix_for_keyring_kind (FwupdKeyringKind keyring_kind)
 	return NULL;
 }
 
-static SoupURI *
+static gchar *
 fwupd_remote_build_uri (FwupdRemote *self, const gchar *url, GError **error)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE (self);
-	SoupURI *uri;
-
-	g_return_val_if_fail (FWUPD_IS_REMOTE (self), NULL);
-	g_return_val_if_fail (url != NULL, NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+#ifdef HAVE_LIBCURL_7_62_0
+	g_autoptr(curlptr) tmp_uri = NULL;
+	g_autoptr(CURLU) uri = curl_url ();
 
 	/* create URI, substituting if required */
 	if (priv->firmware_base_uri != NULL) {
-		g_autoptr(SoupURI) uri_tmp = NULL;
 		g_autofree gchar *basename = NULL;
-		g_autofree gchar *url2 = NULL;
-		uri_tmp = soup_uri_new (url);
-		if (uri_tmp == NULL) {
+		g_autofree gchar *path_new = NULL;
+		g_autoptr(curlptr) path = NULL;
+		g_autoptr(CURLU) uri_tmp = curl_url ();
+		if (curl_url_set (uri_tmp, CURLUPART_URL, url, 0) != CURLUE_OK) {
 			g_set_error (error,
 				     FWUPD_ERROR,
 				     FWUPD_ERROR_INVALID_FILE,
-				     "Failed to parse URI '%s'", url);
-			return NULL;
-		}
-		basename = g_path_get_basename (soup_uri_get_path (uri_tmp));
-		url2 = g_build_filename (priv->firmware_base_uri, basename, NULL);
-		uri = soup_uri_new (url2);
-		if (uri == NULL) {
-			g_set_error (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INVALID_FILE,
-				     "Failed to parse URI '%s'", url2);
+				     "Failed to parse url '%s'", url);
 			return NULL;
 		}
+		curl_url_get (uri_tmp, CURLUPART_PATH, &path, 0);
+		basename = g_path_get_basename (path);
+		path_new = g_build_filename (priv->firmware_base_uri, basename, NULL);
+		curl_url_set (uri, CURLUPART_URL, path_new, 0);
 
 	/* use the base URI of the metadata to build the full path */
 	} else if (g_strstr_len (url, -1, "/") == NULL) {
 		g_autofree gchar *basename = NULL;
-		g_autofree gchar *path = NULL;
-		uri = soup_uri_new (priv->metadata_uri);
-		if (uri == NULL) {
+		g_autofree gchar *path_new = NULL;
+		g_autoptr(curlptr) path = NULL;
+		if (curl_url_set (uri, CURLUPART_URL, priv->metadata_uri, 0) != CURLUE_OK) {
 			g_set_error (error,
 				     FWUPD_ERROR,
 				     FWUPD_ERROR_INVALID_FILE,
-				     "Failed to parse metadata URI '%s'", url);
+				     "Failed to parse url '%s'",
+				     priv->metadata_uri);
 			return NULL;
 		}
-		basename = g_path_get_dirname (soup_uri_get_path (uri));
-		path = g_build_filename (basename, url, NULL);
-		soup_uri_set_path (uri, path);
+		curl_url_get (uri, CURLUPART_PATH, &path, 0);
+		basename = g_path_get_dirname (path);
+		path_new = g_build_filename (basename, url, NULL);
+		curl_url_set (uri, CURLUPART_URL, path_new, 0);
 
 	/* a normal URI */
 	} else {
-		uri = soup_uri_new (url);
-		if (uri == NULL) {
+		if (curl_url_set (uri, CURLUPART_URL, url, 0) != CURLUE_OK) {
 			g_set_error (error,
 				     FWUPD_ERROR,
 				     FWUPD_ERROR_INVALID_FILE,
@@ -223,10 +234,22 @@ fwupd_remote_build_uri (FwupdRemote *self, const gchar *url, GError **error)
 
 	/* set the username and password */
 	if (priv->username != NULL)
-		soup_uri_set_user (uri, priv->username);
+		curl_url_set (uri, CURLUPART_USER, priv->username, 0);
 	if (priv->password != NULL)
-		soup_uri_set_password (uri, priv->password);
-	return uri;
+		curl_url_set (uri, CURLUPART_PASSWORD, priv->password, 0);
+	curl_url_get (uri, CURLUPART_URL, &tmp_uri, 0);
+	return g_strdup (tmp_uri);
+#else
+	if (priv->firmware_base_uri != NULL) {
+		g_autofree gchar *basename = g_path_get_basename (url);
+		return g_build_filename (priv->firmware_base_uri, basename, NULL);
+	}
+	if (g_strstr_len (url, -1, "/") == NULL) {
+		g_autofree gchar *basename = g_path_get_dirname (priv->metadata_uri);
+		return g_build_filename (basename, url, NULL);
+	}
+	return g_strdup (url);
+#endif
 }
 
 /* note, this has to be set before username and password */
@@ -235,12 +258,6 @@ fwupd_remote_set_metadata_uri (FwupdRemote *self, const gchar *metadata_uri)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE (self);
 	const gchar *suffix;
-	g_autoptr(SoupURI) uri = NULL;
-
-	/* build the URI */
-	uri = soup_uri_new (metadata_uri);
-	if (uri == NULL)
-		return;
 
 	/* save this so we can export the object as a GVariant */
 	priv->metadata_uri = g_strdup (metadata_uri);
@@ -266,6 +283,13 @@ fwupd_remote_set_report_uri (FwupdRemote *self, const gchar *report_uri)
 	priv->report_uri = g_strdup (report_uri);
 }
 
+static void
+fwupd_remote_set_security_report_uri (FwupdRemote *self, const gchar *security_report_uri)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE (self);
+	priv->security_report_uri = g_strdup (security_report_uri);
+}
+
 /**
  * fwupd_remote_kind_from_string:
  * @kind: a string, e.g. `download`
@@ -359,6 +383,7 @@ fwupd_remote_load_from_filename (FwupdRemote *self,
 	g_autofree gchar *order_after = NULL;
 	g_autofree gchar *order_before = NULL;
 	g_autofree gchar *report_uri = NULL;
+	g_autofree gchar *security_report_uri = NULL;
 	g_autoptr(GKeyFile) kf = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_REMOTE (self), FALSE);
@@ -426,8 +451,14 @@ fwupd_remote_load_from_filename (FwupdRemote *self,
 	if (report_uri != NULL && report_uri[0] != '\0')
 		fwupd_remote_set_report_uri (self, report_uri);
 
+	/* security reporting is optional */
+	security_report_uri = g_key_file_get_string (kf, group, "SecurityReportURI", NULL);
+	if (security_report_uri != NULL && security_report_uri[0] != '\0')
+		fwupd_remote_set_security_report_uri (self, security_report_uri);
+
 	/* automatic report uploading */
 	priv->automatic_reports = g_key_file_get_boolean (kf, group, "AutomaticReports", NULL);
+	priv->automatic_security_reports = g_key_file_get_boolean (kf, group, "AutomaticSecurityReports", NULL);
 
 	/* DOWNLOAD-type remotes */
 	if (priv->kind == FWUPD_REMOTE_KIND_DOWNLOAD) {
@@ -866,10 +897,10 @@ fwupd_remote_get_checksum (FwupdRemote *self)
 gchar *
 fwupd_remote_build_firmware_uri (FwupdRemote *self, const gchar *url, GError **error)
 {
-	g_autoptr(SoupURI) uri = fwupd_remote_build_uri (self, url, error);
-	if (uri == NULL)
-		return NULL;
-	return soup_uri_to_string (uri, FALSE);
+	g_return_val_if_fail (FWUPD_IS_REMOTE (self), NULL);
+	g_return_val_if_fail (url != NULL, NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	return fwupd_remote_build_uri (self, url, error);
 }
 
 /**
@@ -890,6 +921,24 @@ fwupd_remote_get_report_uri (FwupdRemote *self)
 	return priv->report_uri;
 }
 
+/**
+ * fwupd_remote_get_security_report_uri:
+ * @self: A #FwupdRemote
+ *
+ * Gets the URI for the security report.
+ *
+ * Returns: (transfer none): a URI, or %NULL for invalid.
+ *
+ * Since: 1.5.0
+ **/
+const gchar *
+fwupd_remote_get_security_report_uri (FwupdRemote *self)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE (self);
+	g_return_val_if_fail (FWUPD_IS_REMOTE (self), NULL);
+	return priv->security_report_uri;
+}
+
 /**
  * fwupd_remote_get_metadata_uri:
  * @self: A #FwupdRemote
@@ -959,6 +1008,9 @@ fwupd_remote_load_signature_jcat (FwupdRemote *self, JcatFile *jcat_file, GError
  *
  * Parses the signature, updating the metadata URI as appropriate.
  *
+ * This can only be called for remotes with `Keyring=jcat` which is
+ * the default for most remotes.
+ *
  * Returns: %TRUE for success
  *
  * Since: 1.4.5
@@ -966,6 +1018,7 @@ fwupd_remote_load_signature_jcat (FwupdRemote *self, JcatFile *jcat_file, GError
 gboolean
 fwupd_remote_load_signature_bytes (FwupdRemote *self, GBytes *bytes, GError **error)
 {
+	FwupdRemotePrivate *priv = GET_PRIVATE (self);
 	g_autoptr(GInputStream) istr = NULL;
 	g_autoptr(JcatFile) jcat_file = jcat_file_new ();
 
@@ -973,6 +1026,15 @@ fwupd_remote_load_signature_bytes (FwupdRemote *self, GBytes *bytes, GError **er
 	g_return_val_if_fail (bytes != NULL, FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+	/* sanity check */
+	if (priv->keyring_kind != FWUPD_KEYRING_KIND_JCAT) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "only supported for JCat remotes");
+		return FALSE;
+	}
+
 	istr = g_memory_input_stream_new_from_bytes (bytes);
 	if (!jcat_file_import_stream (jcat_file, istr, JCAT_IMPORT_FLAG_NONE, NULL, error))
 		return FALSE;
@@ -1080,6 +1142,24 @@ fwupd_remote_get_automatic_reports (FwupdRemote *self)
 	return priv->automatic_reports;
 }
 
+/**
+ * fwupd_remote_get_automatic_security_reports:
+ * @self: A #FwupdRemote
+ *
+ * Gets if security reports should be automatically uploaded to this remote
+ *
+ * Returns: a #TRUE if the remote should have reports uploaded automatically
+ *
+ * Since: 1.5.0
+ **/
+gboolean
+fwupd_remote_get_automatic_security_reports (FwupdRemote *self)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE (self);
+	g_return_val_if_fail (FWUPD_IS_REMOTE (self), FALSE);
+	return priv->automatic_security_reports;
+}
+
 /**
  * fwupd_remote_get_approval_required:
  * @self: A #FwupdRemote
@@ -1144,6 +1224,8 @@ fwupd_remote_set_from_variant_iter (FwupdRemote *self, GVariantIter *iter)
 			fwupd_remote_set_filename_source (self, g_variant_get_string (value, NULL));
 		if (g_strcmp0 (key, "ReportUri") == 0)
 			fwupd_remote_set_report_uri (self, g_variant_get_string (value, NULL));
+		if (g_strcmp0 (key, "SecurityReportUri") == 0)
+			fwupd_remote_set_security_report_uri (self, g_variant_get_string (value, NULL));
 	}
 	while (g_variant_iter_loop (iter3, "{sv}", &key, &value)) {
 		if (g_strcmp0 (key, "Username") == 0) {
@@ -1168,6 +1250,8 @@ fwupd_remote_set_from_variant_iter (FwupdRemote *self, GVariantIter *iter)
 			fwupd_remote_set_firmware_base_uri (self, g_variant_get_string (value, NULL));
 		} else if (g_strcmp0 (key, "AutomaticReports") == 0) {
 			priv->automatic_reports = g_variant_get_boolean (value);
+		} else if (g_strcmp0 (key, "AutomaticSecurityReports") == 0) {
+			priv->automatic_security_reports = g_variant_get_boolean (value);
 		}
 	}
 }
@@ -1224,6 +1308,10 @@ fwupd_remote_to_variant (FwupdRemote *self)
 		g_variant_builder_add (&builder, "{sv}", "ReportUri",
 				       g_variant_new_string (priv->report_uri));
 	}
+	if (priv->security_report_uri != NULL) {
+		g_variant_builder_add (&builder, "{sv}", "SecurityReportUri",
+				       g_variant_new_string (priv->security_report_uri));
+	}
 	if (priv->firmware_base_uri != NULL) {
 		g_variant_builder_add (&builder, "{sv}", "FirmwareBaseUri",
 				       g_variant_new_string (priv->firmware_base_uri));
@@ -1262,6 +1350,8 @@ fwupd_remote_to_variant (FwupdRemote *self)
 			       g_variant_new_boolean (priv->approval_required));
 	g_variant_builder_add (&builder, "{sv}", "AutomaticReports",
 			       g_variant_new_boolean (priv->automatic_reports));
+	g_variant_builder_add (&builder, "{sv}", "AutomaticSecurityReports",
+			       g_variant_new_boolean (priv->automatic_security_reports));
 	return g_variant_new ("a{sv}", &builder);
 }
 
@@ -1285,6 +1375,9 @@ fwupd_remote_get_property (GObject *obj, guint prop_id,
 	case PROP_AUTOMATIC_REPORTS:
 		g_value_set_boolean (value, priv->automatic_reports);
 		break;
+	case PROP_AUTOMATIC_SECURITY_REPORTS:
+		g_value_set_boolean (value, priv->automatic_security_reports);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 		break;
@@ -1311,6 +1404,9 @@ fwupd_remote_set_property (GObject *obj, guint prop_id,
 	case PROP_AUTOMATIC_REPORTS:
 		priv->automatic_reports = g_value_get_boolean (value);
 		break;
+	case PROP_AUTOMATIC_SECURITY_REPORTS:
+		priv->automatic_security_reports = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 		break;
@@ -1371,6 +1467,17 @@ fwupd_remote_class_init (FwupdRemoteClass *klass)
 				      FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_NAME);
 	g_object_class_install_property (object_class, PROP_AUTOMATIC_REPORTS, pspec);
 
+	/**
+	* FwupdRemote:automatic-security-reports:
+	*
+	* The behavior for auto-uploading security reports.
+	*
+	* Since: 1.5.0
+	*/
+	pspec = g_param_spec_boolean ("automatic-security-reports", NULL, NULL,
+				      FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_NAME);
+	g_object_class_install_property (object_class, PROP_AUTOMATIC_SECURITY_REPORTS, pspec);
+
 }
 
 static void
@@ -1389,6 +1496,7 @@ fwupd_remote_finalize (GObject *obj)
 	g_free (priv->metadata_uri_sig);
 	g_free (priv->firmware_base_uri);
 	g_free (priv->report_uri);
+	g_free (priv->security_report_uri);
 	g_free (priv->username);
 	g_free (priv->password);
 	g_free (priv->title);
