@@ -12,6 +12,7 @@
 #endif
 
 #include "fwupd-client.h"
+#include "fwupd-client-sync.h"
 #include "fwupd-common.h"
 #include "fwupd-enums.h"
 #include "fwupd-error.h"
@@ -147,6 +148,12 @@ fwupd_enums_func (void)
 			break;
 		g_assert_cmpint (fwupd_device_flag_from_string (tmp), ==, i);
 	}
+	for (guint64 i = 1; i < FWUPD_PLUGIN_FLAG_UNKNOWN; i *= 2) {
+		const gchar *tmp = fwupd_plugin_flag_to_string (i);
+		if (tmp == NULL)
+			break;
+		g_assert_cmpint (fwupd_plugin_flag_from_string (tmp), ==, i);
+	}
 }
 
 static void
@@ -330,11 +337,7 @@ fwupd_device_func (void)
 	fwupd_device_add_guid (dev, "00000000-0000-0000-0000-000000000000");
 	fwupd_device_add_icon (dev, "input-gaming");
 	fwupd_device_add_icon (dev, "input-mouse");
-	fwupd_device_add_flag (dev, FWUPD_DEVICE_FLAG_UPDATABLE |
-				    FWUPD_DEVICE_FLAG_REQUIRE_AC);
-	g_assert_true (fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_REQUIRE_AC));
-	g_assert_true (fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_UPDATABLE));
-	g_assert_false (fwupd_device_has_flag (dev, FWUPD_DEVICE_FLAG_HISTORICAL));
+	fwupd_device_add_flag (dev, FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	rel = fwupd_release_new ();
 	fwupd_release_add_flag (rel, FWUPD_RELEASE_FLAG_TRUSTED_PAYLOAD);
 	fwupd_release_add_checksum (rel, "deadbeef");
