@@ -444,10 +444,8 @@ fu_common_version_verify_format (const gchar *version,
 		return TRUE;
 
 	/* nothing we can check for */
-	if (fmt == FWUPD_VERSION_FORMAT_UNKNOWN) {
-		g_debug ("not checking %s as no version format set", version);
+	if (fmt == FWUPD_VERSION_FORMAT_UNKNOWN)
 		return TRUE;
-	}
 
 	/* check the base format */
 	fmt_guess = fu_common_version_guess_format (version);
@@ -484,6 +482,13 @@ fu_common_vercmp_full (const gchar *version_a,
 {
 	if (fmt == FWUPD_VERSION_FORMAT_PLAIN)
 		return g_strcmp0 (version_a, version_b);
+	if (fmt == FWUPD_VERSION_FORMAT_HEX) {
+		g_autofree gchar *hex_a = NULL;
+		g_autofree gchar *hex_b = NULL;
+		hex_a = fu_common_version_parse_from_format (version_a, fmt);
+		hex_b = fu_common_version_parse_from_format (version_b, fmt);
+		return fu_common_vercmp (hex_a, hex_b);
+	}
 	return fu_common_vercmp (version_a, version_b);
 }
 
@@ -509,7 +514,7 @@ fu_common_vercmp (const gchar *version_a, const gchar *version_b)
 	if (version_a == NULL || version_b == NULL)
 		return G_MAXINT;
 
-	/* optimisation */
+	/* optimization */
 	if (g_strcmp0 (version_a, version_b) == 0)
 		return 0;
 
