@@ -16,12 +16,11 @@ gboolean	 fu_device_has_parent_guid		(FuDevice	*self,
 							 const gchar	*guid);
 void		 fu_device_set_parent			(FuDevice	*self,
 							 FuDevice	*parent);
-guint		 fu_device_get_order			(FuDevice	*self);
+gint		 fu_device_get_order			(FuDevice	*self);
 void		 fu_device_set_order			(FuDevice	*self,
-							 guint		 order);
+							 gint		 order);
 void		 fu_device_set_alternate		(FuDevice	*self,
 							 FuDevice	*alternate);
-GType		 fu_device_get_specialized_gtype	(FuDevice	*self);
 gboolean	 fu_device_ensure_id			(FuDevice	*self,
 							 GError		**error);
 void		 fu_device_incorporate_from_component	(FuDevice	*device,
@@ -29,3 +28,5 @@ void		 fu_device_incorporate_from_component	(FuDevice	*device,
 void		 fu_device_convert_instance_ids		(FuDevice	*self);
 gchar		*fu_device_get_guids_as_str		(FuDevice	*self);
 GPtrArray	*fu_device_get_possible_plugins		(FuDevice	*self);
+void		 fu_device_add_possible_plugin		(FuDevice	*self,
+							 const gchar	*plugin);
