@@ -11,7 +11,7 @@
 #include <fcntl.h>
 #include <string.h>
 #ifdef HAVE_ERRNO_H
-#include <sys/errno.h>
+#include <errno.h>
 #endif
 #ifdef HAVE_IOCTL_H
 #include <sys/ioctl.h>
@@ -39,8 +39,11 @@ typedef struct
 	GUdevDevice		*udev_device;
 	guint32			 vendor;
 	guint32			 model;
+	guint32			 subsystem_vendor;
+	guint32			 subsystem_model;
 	guint8			 revision;
 	gchar			*subsystem;
+	gchar			*driver;
 	gchar			*device_file;
 	gint			 fd;
 	FuUdevDeviceFlags	 flags;
@@ -52,6 +55,7 @@ enum {
 	PROP_0,
 	PROP_UDEV_DEVICE,
 	PROP_SUBSYSTEM,
+	PROP_DRIVER,
 	PROP_DEVICE_FILE,
 	PROP_LAST
 };
@@ -76,8 +80,11 @@ static guint signals[SIGNAL_LAST] = { 0 };
 void
 fu_udev_device_emit_changed (FuUdevDevice *self)
 {
+	g_autoptr(GError) error = NULL;
 	g_return_if_fail (FU_IS_UDEV_DEVICE (self));
 	g_debug ("FuUdevDevice emit changed");
+	if (!fu_device_rescan (FU_DEVICE (self), &error))
+		g_debug ("%s", error->message);
 	g_signal_emit (self, signals[SIGNAL_CHANGED], 0);
 }
 
@@ -145,6 +152,11 @@ fu_udev_device_to_string (FuDevice *device, guint idt, GString *str)
 	if (priv->udev_device != NULL) {
 		fu_common_string_append_kv (str, idt, "SysfsPath",
 					    g_udev_device_get_sysfs_path (priv->udev_device));
+		fu_common_string_append_kv (str, idt, "Subsystem", priv->subsystem);
+		if (priv->driver != NULL)
+			fu_common_string_append_kv (str, idt, "Driver", priv->driver);
+		if (priv->device_file != NULL)
+			fu_common_string_append_kv (str, idt, "DeviceFile", priv->device_file);
 	}
 	if (g_getenv ("FU_UDEV_DEVICE_DEBUG") != NULL) {
 		g_autoptr(GUdevDevice) udev_parent = NULL;
@@ -176,6 +188,20 @@ fu_udev_device_set_subsystem (FuUdevDevice *self, const gchar *subsystem)
 	g_object_notify (G_OBJECT (self), "subsystem");
 }
 
+static void
+fu_udev_device_set_driver (FuUdevDevice *self, const gchar *driver)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+
+	/* not changed */
+	if (g_strcmp0 (priv->driver, driver) == 0)
+		return;
+
+	g_free (priv->driver);
+	priv->driver = g_strdup (driver);
+	g_object_notify (G_OBJECT (self), "driver");
+}
+
 static void
 fu_udev_device_set_device_file (FuUdevDevice *self, const gchar *device_file)
 {
@@ -190,23 +216,59 @@ fu_udev_device_set_device_file (FuUdevDevice *self, const gchar *device_file)
 	g_object_notify (G_OBJECT (self), "device-file");
 }
 
+#ifdef HAVE_GUDEV
 static const gchar *
 fu_udev_device_get_vendor_fallback (GUdevDevice *udev_device)
 {
-#ifdef HAVE_GUDEV
 	const gchar *tmp;
-	tmp = g_udev_device_get_property (udev_device, "FWUPD_VENDOR");
-	if (tmp != NULL)
-		return tmp;
 	tmp = g_udev_device_get_property (udev_device, "ID_VENDOR_FROM_DATABASE");
 	if (tmp != NULL)
 		return tmp;
 	tmp = g_udev_device_get_property (udev_device, "ID_VENDOR");
 	if (tmp != NULL)
 		return tmp;
-#endif
 	return NULL;
 }
+#endif
+
+#ifdef HAVE_GUDEV
+static gboolean
+fu_udev_device_probe_i2c_dev (FuUdevDevice *self, GError **error)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	const gchar *name = g_udev_device_get_sysfs_attr (priv->udev_device, "name");
+	if (name != NULL) {
+		g_autofree gchar *devid = NULL;
+		g_autofree gchar *name_safe = g_strdup (name);
+		g_strdelimit (name_safe, " /\\\"", '-');
+		devid = g_strdup_printf ("I2C\\NAME_%s", name_safe);
+		fu_device_add_instance_id (FU_DEVICE (self), devid);
+	}
+	return TRUE;
+}
+
+static gboolean
+fu_udev_device_probe_serio (FuUdevDevice *self, GError **error)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	const gchar *tmp;
+
+	/* firmware ID */
+	tmp = g_udev_device_get_property (priv->udev_device, "SERIO_FIRMWARE_ID");
+	if (tmp != NULL) {
+		g_autofree gchar *devid = NULL;
+		g_autofree gchar *id_safe = NULL;
+		/* this prefix is not useful */
+		if (g_str_has_prefix (tmp, "PNP: "))
+			tmp += 5;
+		id_safe = g_utf8_strup (tmp, -1);
+		g_strdelimit (id_safe, " /\\\"", '-');
+		devid = g_strdup_printf ("SERIO\\FWID_%s", id_safe);
+		fu_device_add_instance_id (FU_DEVICE (self), devid);
+	}
+	return TRUE;
+}
+#endif
 
 static gboolean
 fu_udev_device_probe (FuDevice *device, GError **error)
@@ -229,6 +291,8 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 	priv->vendor = fu_udev_device_get_sysfs_attr_as_uint32 (priv->udev_device, "vendor");
 	priv->model = fu_udev_device_get_sysfs_attr_as_uint32 (priv->udev_device, "device");
 	priv->revision = fu_udev_device_get_sysfs_attr_as_uint8 (priv->udev_device, "revision");
+	priv->subsystem_vendor = fu_udev_device_get_sysfs_attr_as_uint32 (priv->udev_device, "subsystem_vendor");
+	priv->subsystem_model = fu_udev_device_get_sysfs_attr_as_uint32 (priv->udev_device, "subsystem_device");
 
 #ifdef HAVE_GUDEV
 	/* fallback to the parent */
@@ -239,6 +303,8 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 		priv->vendor = fu_udev_device_get_sysfs_attr_as_uint32 (udev_parent, "vendor");
 		priv->model = fu_udev_device_get_sysfs_attr_as_uint32 (udev_parent, "device");
 		priv->revision = fu_udev_device_get_sysfs_attr_as_uint8 (udev_parent, "revision");
+		priv->subsystem_vendor = fu_udev_device_get_sysfs_attr_as_uint32 (udev_parent, "subsystem_vendor");
+		priv->subsystem_model = fu_udev_device_get_sysfs_attr_as_uint32 (udev_parent, "subsystem_device");
 	}
 
 	/* hidraw helpfully encodes the information in a different place */
@@ -286,9 +352,7 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 
 	/* set model */
 	if (fu_device_get_name (device) == NULL) {
-		tmp = g_udev_device_get_property (priv->udev_device, "FWUPD_MODEL");
-		if (tmp == NULL)
-			tmp = g_udev_device_get_property (priv->udev_device, "ID_MODEL_FROM_DATABASE");
+		tmp = g_udev_device_get_property (priv->udev_device, "ID_MODEL_FROM_DATABASE");
 		if (tmp == NULL)
 			tmp = g_udev_device_get_property (priv->udev_device, "ID_MODEL");
 		if (tmp == NULL)
@@ -304,7 +368,7 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 			fu_device_set_vendor (device, tmp);
 	}
 
-	/* try harder to find a vendor name the user will recognise */
+	/* try harder to find a vendor name the user will recognize */
 	if (priv->flags & FU_UDEV_DEVICE_FLAG_VENDOR_FROM_PARENT &&
 	    udev_parent != NULL && fu_device_get_vendor (device) == NULL) {
 		g_autoptr(GUdevDevice) device_tmp = g_object_ref (udev_parent);
@@ -348,6 +412,22 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 	}
 
 	/* add GUIDs in order of priority */
+	if (priv->vendor != 0x0000 && priv->model != 0x0000 &&
+	    priv->subsystem_vendor != 0x0000 && priv->subsystem_model != 0x0000) {
+		g_autofree gchar *devid1 = NULL;
+		g_autofree gchar *devid2 = NULL;
+		devid1 = g_strdup_printf ("%s\\VEN_%04X&DEV_%04X&SUBSYS_%04X%04X&REV_%02X",
+					  subsystem,
+					  priv->vendor, priv->model,
+					  priv->subsystem_vendor, priv->subsystem_model,
+					  priv->revision);
+		fu_device_add_instance_id (device, devid1);
+		devid2 = g_strdup_printf ("%s\\VEN_%04X&DEV_%04X&SUBSYS_%04X%04X",
+					  subsystem,
+					  priv->vendor, priv->model,
+					  priv->subsystem_vendor, priv->subsystem_model);
+		fu_device_add_instance_id (device, devid2);
+	}
 	if (priv->vendor != 0x0000 && priv->model != 0x0000) {
 		g_autofree gchar *devid = NULL;
 		devid = g_strdup_printf ("%s\\VEN_%04X&DEV_%04X&REV_%02X",
@@ -368,12 +448,47 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 						FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
 	}
 
+	/* add device class */
+	tmp = g_udev_device_get_sysfs_attr (priv->udev_device, "class");
+	if (tmp != NULL && g_str_has_prefix (tmp, "0x")) {
+		g_autofree gchar *class_id = g_utf8_strup (tmp + 2, -1);
+		g_autofree gchar *devid = NULL;
+		devid = g_strdup_printf ("%s\\VEN_%04X&CLASS_%s",
+					 subsystem,
+					 priv->vendor,
+					 class_id);
+		fu_device_add_instance_id_full (device, devid,
+						FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
+	}
+
+	/* add the driver */
+	if (priv->driver != NULL) {
+		g_autofree gchar *devid = NULL;
+		devid = g_strdup_printf ("%s\\DRIVER_%s",
+					 subsystem,
+					 priv->driver);
+		fu_device_add_instance_id_full (device, devid,
+						FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
+	}
+
 	/* add subsystem to match in plugins */
 	if (subsystem != NULL) {
 		fu_device_add_instance_id_full (device, subsystem,
 						FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
 	}
 
+	/* i2c devices all expose a name */
+	if (g_strcmp0 (g_udev_device_get_subsystem (priv->udev_device), "i2c-dev") == 0) {
+		if (!fu_udev_device_probe_i2c_dev (self, error))
+			return FALSE;
+	}
+
+	/* add firmware_id */
+	if (g_strcmp0 (g_udev_device_get_subsystem (priv->udev_device), "serio") == 0) {
+		if (!fu_udev_device_probe_serio (self, error))
+			return FALSE;
+	}
+
 	/* determine if we're wired internally */
 	parent_i2c = g_udev_device_get_parent_with_subsystem (priv->udev_device,
 							      "i2c", NULL);
@@ -391,28 +506,75 @@ fu_udev_device_probe (FuDevice *device, GError **error)
 	return TRUE;
 }
 
+#ifdef HAVE_GUDEV
+static gchar *
+fu_udev_device_get_miscdev0 (FuUdevDevice *self)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	const gchar *fn;
+	g_autofree gchar *miscdir = NULL;
+	g_autoptr(GDir) dir = NULL;
+
+	miscdir = g_build_filename (g_udev_device_get_sysfs_path (priv->udev_device), "misc", NULL);
+	dir = g_dir_open (miscdir, 0, NULL);
+	if (dir == NULL)
+		return NULL;
+	fn = g_dir_read_name (dir);
+	if (fn == NULL)
+		return NULL;
+	return g_strdup_printf ("/dev/%s", fn);
+}
+#endif
+
 static void
 fu_udev_device_set_dev (FuUdevDevice *self, GUdevDevice *udev_device)
 {
 	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
 #ifdef HAVE_GUDEV
 	const gchar *summary;
-	g_autoptr(GUdevDevice) parent = NULL;
 #endif
 
 	g_return_if_fail (FU_IS_UDEV_DEVICE (self));
 
-	/* set new device */
+#ifdef HAVE_GUDEV
+	/* the net subsystem is not a real hardware class */
+	if (udev_device != NULL &&
+	    g_strcmp0 (g_udev_device_get_subsystem (udev_device), "net") == 0) {
+		g_autoptr(GUdevDevice) udev_device_phys = NULL;
+		udev_device_phys = g_udev_device_get_parent (udev_device);
+		g_set_object (&priv->udev_device, udev_device_phys);
+		fu_device_set_metadata (FU_DEVICE (self),
+					"ParentSubsystem",
+					g_udev_device_get_subsystem (udev_device));
+	} else {
+		g_set_object (&priv->udev_device, udev_device);
+	}
+#else
 	g_set_object (&priv->udev_device, udev_device);
+#endif
+
+	/* set new device */
 	if (priv->udev_device == NULL)
 		return;
 #ifdef HAVE_GUDEV
 	fu_udev_device_set_subsystem (self, g_udev_device_get_subsystem (priv->udev_device));
+	fu_udev_device_set_driver (self, g_udev_device_get_driver (priv->udev_device));
 	fu_udev_device_set_device_file (self, g_udev_device_get_device_file (priv->udev_device));
 
+	/* fall back to the first thing handled by misc drivers */
+	if (priv->device_file == NULL) {
+		/* perhaps we should unconditionally fall back? or perhaps
+		 * require FU_UDEV_DEVICE_FLAG_FALLBACK_MISC... */
+		if (g_strcmp0 (priv->subsystem, "serio") == 0)
+			priv->device_file = fu_udev_device_get_miscdev0 (self);
+		if (priv->device_file != NULL)
+			g_debug ("falling back to misc %s", priv->device_file);
+	}
+
 	/* try to get one line summary */
 	summary = g_udev_device_get_sysfs_attr (priv->udev_device, "description");
 	if (summary == NULL) {
+		g_autoptr(GUdevDevice) parent = NULL;
 		parent = g_udev_device_get_parent (priv->udev_device);
 		if (parent != NULL)
 			summary = g_udev_device_get_sysfs_attr (parent, "description");
@@ -453,6 +615,120 @@ fu_udev_device_get_slot_depth (FuUdevDevice *self, const gchar *subsystem)
 	return 0;
 }
 
+#ifndef _WIN32
+static gchar *
+fu_udev_device_get_bind_id (FuUdevDevice *self)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	if (g_strcmp0 (fu_udev_device_get_subsystem (self), "pci") == 0)
+		return g_strdup (g_udev_device_get_property (priv->udev_device, "PCI_SLOT_NAME"));
+	if (g_strcmp0 (fu_udev_device_get_subsystem (self), "hid") == 0)
+		return g_strdup (g_udev_device_get_property (priv->udev_device, "HID_PHYS"));
+	if (g_strcmp0 (fu_udev_device_get_subsystem (self), "usb") == 0)
+		return g_path_get_basename (g_udev_device_get_sysfs_path (priv->udev_device));
+	return NULL;
+}
+#endif
+
+static gboolean
+fu_udev_device_unbind_driver (FuDevice *device, GError **error)
+{
+#ifndef _WIN32
+	FuUdevDevice *self = FU_UDEV_DEVICE (device);
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	g_autofree gchar *bind_id = NULL;
+	g_autofree gchar *fn = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GOutputStream) stream = NULL;
+
+	/* is already unbound */
+	fn = g_build_filename (g_udev_device_get_sysfs_path (priv->udev_device),
+			       "driver", "unbind", NULL);
+	if (!g_file_test (fn, G_FILE_TEST_EXISTS))
+		return TRUE;
+
+	/* write bus ID to file */
+	bind_id = fu_udev_device_get_bind_id (self);
+	if (bind_id == NULL) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "bind-id not set for subsystem %s",
+			     priv->subsystem);
+		return FALSE;
+	}
+	file = g_file_new_for_path (fn);
+	stream = G_OUTPUT_STREAM (g_file_replace (file, NULL, FALSE,
+				  G_FILE_CREATE_NONE, NULL, error));
+	if (stream == NULL)
+		return FALSE;
+	return g_output_stream_write_all (stream, bind_id, strlen (bind_id),
+					  NULL, NULL, error);
+#else
+	g_set_error_literal (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "driver unbinding not supported on Windows");
+	return FALSE;
+#endif
+}
+
+static gboolean
+fu_udev_device_bind_driver (FuDevice *device,
+			    const gchar *subsystem,
+			    const gchar *driver,
+			    GError **error)
+{
+#ifndef _WIN32
+	FuUdevDevice *self = FU_UDEV_DEVICE (device);
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	g_autofree gchar *bind_id = NULL;
+	g_autofree gchar *driver_safe = g_strdup (driver);
+	g_autofree gchar *fn = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GOutputStream) stream = NULL;
+
+	/* copy the logic from modprobe */
+	g_strdelimit (driver_safe, "-", '_');
+
+	/* driver exists */
+	fn = g_strdup_printf ("/sys/module/%s/drivers/%s:%s/bind",
+			      driver_safe, subsystem, driver_safe);
+	if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "cannot bind with %s:%s",
+			     subsystem, driver);
+		return FALSE;
+	}
+
+	/* write bus ID to file */
+	bind_id = fu_udev_device_get_bind_id (self);
+	if (bind_id == NULL) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "bind-id not set for subsystem %s",
+			     priv->subsystem);
+		return FALSE;
+	}
+	file = g_file_new_for_path (fn);
+	stream = G_OUTPUT_STREAM (g_file_replace (file, NULL, FALSE,
+				  G_FILE_CREATE_NONE, NULL, error));
+	if (stream == NULL)
+		return FALSE;
+	return g_output_stream_write_all (stream, bind_id, strlen (bind_id),
+					  NULL, NULL, error);
+#else
+	g_set_error_literal (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "driver binding not supported on Windows");
+	return FALSE;
+#endif
+}
+
 static void
 fu_udev_device_incorporate (FuDevice *self, FuDevice *donor)
 {
@@ -506,6 +782,24 @@ fu_udev_device_get_subsystem (FuUdevDevice *self)
 	return priv->subsystem;
 }
 
+/**
+ * fu_udev_device_get_driver:
+ * @self: A #FuUdevDevice
+ *
+ * Gets the device driver, e.g. "psmouse".
+ *
+ * Returns: a subsystem, or NULL if unset or invalid
+ *
+ * Since: 1.5.3
+ **/
+const gchar *
+fu_udev_device_get_driver (FuUdevDevice *self)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	g_return_val_if_fail (FU_IS_UDEV_DEVICE (self), NULL);
+	return priv->driver;
+}
+
 /**
  * fu_udev_device_get_device_file:
  * @self: A #FuUdevDevice
@@ -546,6 +840,29 @@ fu_udev_device_get_sysfs_path (FuUdevDevice *self)
 	return NULL;
 }
 
+/**
+ * fu_udev_device_get_number:
+ * @self: A #FuUdevDevice
+ *
+ * Gets the device number, if any.
+ *
+ * Returns: integer, 0 if the data is unavailable, or %G_MAXUINT64 if the
+ * feature is not available
+ *
+ * Since: 1.5.0
+ **/
+guint64
+fu_udev_device_get_number (FuUdevDevice *self)
+{
+#ifdef HAVE_GUDEV
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	g_return_val_if_fail (FU_IS_UDEV_DEVICE (self), 0);
+	if (priv->udev_device != NULL)
+		return fu_common_strtoull (g_udev_device_get_number (priv->udev_device));
+#endif
+	return G_MAXUINT64;
+}
+
 /**
  * fu_udev_device_get_vendor:
  * @self: A #FuUdevDevice
@@ -582,6 +899,42 @@ fu_udev_device_get_model (FuUdevDevice *self)
 	return priv->model;
 }
 
+/**
+ * fu_udev_device_get_subsystem_vendor:
+ * @self: A #FuUdevDevice
+ *
+ * Gets the device subsystem vendor code.
+ *
+ * Returns: a vendor code, or 0 if unset or invalid
+ *
+ * Since: 1.5.0
+ **/
+guint32
+fu_udev_device_get_subsystem_vendor (FuUdevDevice *self)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	g_return_val_if_fail (FU_IS_UDEV_DEVICE (self), 0x0000);
+	return priv->subsystem_vendor;
+}
+
+/**
+ * fu_udev_device_get_subsystem_model:
+ * @self: A #FuUdevDevice
+ *
+ * Gets the device subsystem model code.
+ *
+ * Returns: a vendor code, or 0 if unset or invalid
+ *
+ * Since: 1.5.0
+ **/
+guint32
+fu_udev_device_get_subsystem_model (FuUdevDevice *self)
+{
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	g_return_val_if_fail (FU_IS_UDEV_DEVICE (self), 0x0000);
+	return priv->subsystem_model;
+}
+
 /**
  * fu_udev_device_get_revision:
  * @self: A #FuUdevDevice
@@ -695,6 +1048,7 @@ fu_udev_device_set_physical_id (FuUdevDevice *self, const gchar *subsystems, GEr
 	} else if (g_strcmp0 (subsystem, "usb") == 0 ||
 		   g_strcmp0 (subsystem, "mmc") == 0 ||
 		   g_strcmp0 (subsystem, "i2c") == 0 ||
+		   g_strcmp0 (subsystem, "platform") == 0 ||
 		   g_strcmp0 (subsystem, "scsi") == 0) {
 		tmp = g_udev_device_get_property (udev_device, "DEVPATH");
 		if (tmp == NULL) {
@@ -824,6 +1178,15 @@ fu_udev_device_set_flags (FuUdevDevice *self, FuUdevDeviceFlags flags)
 	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
 	g_return_if_fail (FU_IS_UDEV_DEVICE (self));
 	priv->flags = flags;
+
+#ifdef HAVE_GUDEV
+	/* overwrite */
+	if (flags & FU_UDEV_DEVICE_FLAG_USE_CONFIG) {
+		g_free (priv->device_file);
+		priv->device_file = g_build_filename (g_udev_device_get_sysfs_path (priv->udev_device),
+						      "config", NULL);
+	}
+#endif
 }
 
 static gboolean
@@ -844,6 +1207,10 @@ fu_udev_device_open (FuDevice *device, GError **error)
 		} else {
 			flags = O_RDONLY;
 		}
+#ifdef O_NONBLOCK
+		if (priv->flags & FU_UDEV_DEVICE_FLAG_OPEN_NONBLOCK)
+			flags |= O_NONBLOCK;
+#endif
 		priv->fd = g_open (priv->device_file, flags, 0);
 		if (priv->fd < 0) {
 			g_set_error (error,
@@ -866,6 +1233,40 @@ fu_udev_device_open (FuDevice *device, GError **error)
 	return TRUE;
 }
 
+static gboolean
+fu_udev_device_rescan (FuDevice *device, GError **error)
+{
+#ifdef HAVE_GUDEV
+	FuUdevDevice *self = FU_UDEV_DEVICE (device);
+	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
+	const gchar *sysfs_path;
+	g_autoptr(GUdevClient) udev_client = g_udev_client_new (NULL);
+	g_autoptr(GUdevDevice) udev_device = NULL;
+
+	/* never set */
+	if (priv->udev_device == NULL) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INTERNAL,
+				     "rescan with no previous device");
+		return FALSE;
+	}
+	sysfs_path = g_udev_device_get_sysfs_path (priv->udev_device);
+	udev_device = g_udev_client_query_by_sysfs_path (udev_client, sysfs_path);
+	if (udev_device == NULL) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_INTERNAL,
+			     "rescan could not find device %s",
+			     sysfs_path);
+		return FALSE;
+	}
+	fu_udev_device_set_dev (self, udev_device);
+	fu_device_probe_invalidate (device);
+#endif
+	return fu_device_probe (device, error);
+}
+
 static gboolean
 fu_udev_device_close (FuDevice *device, GError **error)
 {
@@ -970,7 +1371,6 @@ fu_udev_device_pread_full (FuUdevDevice *self, goffset port,
 	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
 
 	g_return_val_if_fail (FU_IS_UDEV_DEVICE (self), FALSE);
-	g_return_val_if_fail (port != 0x0, FALSE);
 	g_return_val_if_fail (buf != NULL, FALSE);
 	g_return_val_if_fail (priv->fd > 0, FALSE);
 
@@ -1016,7 +1416,6 @@ fu_udev_device_pwrite_full (FuUdevDevice *self, goffset port,
 	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
 
 	g_return_val_if_fail (FU_IS_UDEV_DEVICE (self), FALSE);
-	g_return_val_if_fail (port != 0x0, FALSE);
 	g_return_val_if_fail (priv->fd > 0, FALSE);
 
 #ifdef HAVE_PWRITE
@@ -1262,6 +1661,9 @@ fu_udev_device_get_property (GObject *object, guint prop_id,
 	case PROP_SUBSYSTEM:
 		g_value_set_string (value, priv->subsystem);
 		break;
+	case PROP_DRIVER:
+		g_value_set_string (value, priv->driver);
+		break;
 	case PROP_DEVICE_FILE:
 		g_value_set_string (value, priv->device_file);
 		break;
@@ -1283,6 +1685,9 @@ fu_udev_device_set_property (GObject *object, guint prop_id,
 	case PROP_SUBSYSTEM:
 		fu_udev_device_set_subsystem (self, g_value_get_string (value));
 		break;
+	case PROP_DRIVER:
+		fu_udev_device_set_driver (self, g_value_get_string (value));
+		break;
 	case PROP_DEVICE_FILE:
 		fu_udev_device_set_device_file (self, g_value_get_string (value));
 		break;
@@ -1299,6 +1704,7 @@ fu_udev_device_finalize (GObject *object)
 	FuUdevDevicePrivate *priv = GET_PRIVATE (self);
 
 	g_free (priv->subsystem);
+	g_free (priv->driver);
 	g_free (priv->device_file);
 	if (priv->udev_device != NULL)
 		g_object_unref (priv->udev_device);
@@ -1327,10 +1733,13 @@ fu_udev_device_class_init (FuUdevDeviceClass *klass)
 	object_class->get_property = fu_udev_device_get_property;
 	object_class->set_property = fu_udev_device_set_property;
 	device_class->probe = fu_udev_device_probe;
+	device_class->rescan = fu_udev_device_rescan;
 	device_class->incorporate = fu_udev_device_incorporate;
 	device_class->open = fu_udev_device_open;
 	device_class->close = fu_udev_device_close;
 	device_class->to_string = fu_udev_device_to_string;
+	device_class->bind_driver = fu_udev_device_bind_driver;
+	device_class->unbind_driver = fu_udev_device_unbind_driver;
 
 	signals[SIGNAL_CHANGED] =
 		g_signal_new ("changed",
@@ -1351,6 +1760,12 @@ fu_udev_device_class_init (FuUdevDeviceClass *klass)
 				     G_PARAM_STATIC_NAME);
 	g_object_class_install_property (object_class, PROP_SUBSYSTEM, pspec);
 
+	pspec = g_param_spec_string ("driver", NULL, NULL,
+				     NULL,
+				     G_PARAM_READWRITE |
+				     G_PARAM_STATIC_NAME);
+	g_object_class_install_property (object_class, PROP_DRIVER, pspec);
+
 	pspec = g_param_spec_string ("device-file", NULL, NULL,
 				     NULL,
 				     G_PARAM_READWRITE |
