@@ -42,8 +42,8 @@ typedef enum {
 	CH_ERROR_SELF_TEST_ADC_VDD,
 	CH_ERROR_SELF_TEST_ADC_VSS,
 	CH_ERROR_SELF_TEST_ADC_VREF,
-	CH_ERROR_I2C_SLAVE_ADDRESS,
-	CH_ERROR_I2C_SLAVE_CONFIG,
+	CH_ERROR_I2C_TARGET_ADDRESS,
+	CH_ERROR_I2C_TARGET_CONFIG,
 	CH_ERROR_SELF_TEST_EEPROM,
 	CH_ERROR_LAST
 } ChError;
