@@ -1270,7 +1270,7 @@ dfu_device_probe (FuUsbDevice *device, GError **error)
 	/* hardware rom Jabra literally reboots if you try to retry a failed
 	 * write -- there's no way to avoid blocking the daemon like this... */
 	if (fu_device_has_custom_flag (FU_DEVICE (device), "attach-extra-reset"))
-		g_usleep (10 * G_USEC_PER_SEC);
+		fu_device_sleep_with_progress (FU_DEVICE (self), 10); /* seconds */
 
 	/* success */
 	return TRUE;
@@ -1679,12 +1679,20 @@ dfu_device_error_fixup (DfuDevice *device, GError **error)
 	}
 }
 
-static FuFirmware *
-dfu_device_read_firmware (FuDevice *device, GError **error)
+static GBytes *
+dfu_device_dump_firmware (FuDevice *device, GError **error)
 {
 	DfuDevice *self = DFU_DEVICE (device);
 	g_autoptr(DfuFirmware) dfu_firmware = NULL;
-	g_autoptr(GBytes) fw = NULL;
+	g_autoptr(FuDeviceLocker) locker = NULL;
+
+	/* require detach -> attach */
+	locker = fu_device_locker_new_full (device,
+					    (FuDeviceLockerFunc) fu_device_detach,
+					    (FuDeviceLockerFunc) fu_device_attach,
+					    error);
+	if (locker == NULL)
+		return NULL;
 
 	/* get data from hardware */
 	g_debug ("uploading from device->host");
@@ -1697,8 +1705,7 @@ dfu_device_read_firmware (FuDevice *device, GError **error)
 		return NULL;
 
 	/* get the checksum */
-	fw = dfu_firmware_write_data (dfu_firmware, error);
-	return fu_firmware_new_from_bytes (fw);
+	return dfu_firmware_write_data (dfu_firmware, error);
 }
 
 static gboolean
@@ -1719,7 +1726,7 @@ dfu_device_write_firmware (FuDevice *device,
 	if (!dfu_device_refresh_and_clear (self, error))
 		return FALSE;
 
-	if (flags & FWUPD_INSTALL_FLAG_FORCE) {
+	if (flags & FWUPD_INSTALL_FLAG_IGNORE_VID_PID) {
 		transfer_flags |= DFU_TARGET_TRANSFER_FLAG_WILDCARD_VID;
 		transfer_flags |= DFU_TARGET_TRANSFER_FLAG_WILDCARD_PID;
 	}
@@ -1825,7 +1832,7 @@ dfu_device_class_init (DfuDeviceClass *klass)
 	FuUsbDeviceClass *klass_usb_device = FU_USB_DEVICE_CLASS (klass);
 	klass_device->set_quirk_kv = dfu_device_set_quirk_kv;
 	klass_device->to_string = dfu_device_to_string;
-	klass_device->read_firmware = dfu_device_read_firmware;
+	klass_device->dump_firmware = dfu_device_dump_firmware;
 	klass_device->write_firmware = dfu_device_write_firmware;
 	klass_device->attach = dfu_device_attach;
 	klass_device->detach = dfu_device_detach;
