@@ -49,8 +49,8 @@ gboolean
 fu_plugin_startup (FuPlugin *plugin, GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
+	gboolean ca_check;
 	g_autofree gchar *redfish_uri = NULL;
-	g_autofree gchar *ca_check = NULL;
 	g_autoptr(GBytes) smbios_data = NULL;
 
 	/* optional */
@@ -107,12 +107,8 @@ fu_plugin_startup (FuPlugin *plugin, GError **error)
 		}
 	}
 
-	ca_check = fu_plugin_get_config_value (plugin, "CACheck");
-	if (ca_check != NULL && g_ascii_strcasecmp (ca_check, "false") == 0)
-		fu_redfish_client_set_cacheck (data->client, FALSE);
-	else
-		fu_redfish_client_set_cacheck (data->client, TRUE);
-
+	ca_check = fu_plugin_get_config_value_boolean (plugin, "CACheck");
+	fu_redfish_client_set_cacheck (data->client, ca_check);
 	return fu_redfish_client_setup (data->client, smbios_data, error);
 }
 
