@@ -10,26 +10,6 @@
 
 #include "fu-redfish-common.h"
 
-GBytes *
-fu_redfish_common_get_evivar_raw (efi_guid_t guid, const gchar *name, GError **error)
-{
-	gsize sz = 0;
-	guint32 attribs = 0;
-	guint8 *data = NULL;
-
-	if (efi_get_variable (guid, name, &data, &sz, &attribs) < 0) {
-		g_autofree gchar *guid_str = NULL;
-		efi_guid_to_str (&guid, &guid_str);
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_FILE,
-			     "failed to get efivar for %s %s",
-			     guid_str, name);
-		return NULL;
-	}
-	return g_bytes_new_take (data, sz);
-}
-
 gchar *
 fu_redfish_common_buffer_to_ipv4 (const guint8 *buffer)
 {
