@@ -417,18 +417,35 @@ fu_plugin_superio_fix_signature (FuSuperioDevice *self, GBytes *fw, GError **err
 	return g_bytes_new_take (g_steal_pointer (&buf2), sz);
 }
 
+static GBytes *
+fu_superio_it89_device_dump_firmware (FuDevice *device, GError **error)
+{
+	FuSuperioDevice *self = FU_SUPERIO_DEVICE (device);
+	guint64 fwsize = fu_device_get_firmware_size_min (device);
+	g_autoptr(FuDeviceLocker) locker = NULL;
+
+	/* require detach -> attach */
+	locker = fu_device_locker_new_full (device,
+					    (FuDeviceLockerFunc) fu_device_detach,
+					    (FuDeviceLockerFunc) fu_device_attach,
+					    error);
+	if (locker == NULL)
+		return NULL;
+
+	fu_device_set_status (device, FWUPD_STATUS_DEVICE_READ);
+	return fu_superio_it89_device_read_addr (self, 0x0, fwsize,
+						 fu_superio_it89_device_progress_cb,
+						 error);
+}
+
 static FuFirmware *
 fu_superio_it89_device_read_firmware (FuDevice *device, GError **error)
 {
 	FuSuperioDevice *self = FU_SUPERIO_DEVICE (device);
-	guint64 fwsize = fu_device_get_firmware_size_min (device);
 	g_autoptr(GBytes) blob = NULL;
 	g_autoptr(GBytes) fw = NULL;
 
-	fu_device_set_status (device, FWUPD_STATUS_DEVICE_READ);
-	blob = fu_superio_it89_device_read_addr (self, 0x0, fwsize,
-						 fu_superio_it89_device_progress_cb,
-						 error);
+	blob = fu_superio_it89_device_dump_firmware (device, error);
 	fw = fu_plugin_superio_fix_signature (self, blob, error);
 	return fu_firmware_new_from_bytes (fw);
 }
@@ -483,7 +500,7 @@ fu_superio_it89_device_check_eflash (FuSuperioDevice *self, GError **error)
 	fw = fu_superio_it89_device_read_addr (self, fwsize - sigsz,
 					       sigsz, NULL, error);
 	if (fw == NULL) {
-		g_prefix_error (error, "failed to read signature bytes");
+		g_prefix_error (error, "failed to read signature bytes: ");
 		return FALSE;
 	}
 
@@ -517,7 +534,7 @@ fu_superio_it89_device_write_chunk (FuSuperioDevice *self, FuChunk *chk, GError
 
 	/* erase page */
 	if (!fu_superio_it89_device_erase_addr (self, chk->address, error)) {
-		g_prefix_error (error, "failed to erase @0x%04x", (guint) chk->address);
+		g_prefix_error (error, "failed to erase @0x%04x: ", (guint) chk->address);
 		return FALSE;
 	}
 
@@ -527,7 +544,7 @@ fu_superio_it89_device_write_chunk (FuSuperioDevice *self, FuChunk *chk, GError
 						error);
 	if (fw1 == NULL) {
 		g_prefix_error (error, "failed to read erased "
-				"bytes @0x%04x", (guint) chk->address);
+				"bytes @0x%04x: ", (guint) chk->address);
 		return FALSE;
 	}
 	if (!fu_common_bytes_is_empty (fw1)) {
@@ -545,7 +562,7 @@ fu_superio_it89_device_write_chunk (FuSuperioDevice *self, FuChunk *chk, GError
 
 	/* write page */
 	if (!fu_superio_it89_device_write_addr (self, chk->address, fw2, error)) {
-		g_prefix_error (error, "failed to write @0x%04x", (guint) chk->address);
+		g_prefix_error (error, "failed to write @0x%04x: ", (guint) chk->address);
 		return FALSE;
 	}
 
@@ -555,11 +572,11 @@ fu_superio_it89_device_write_chunk (FuSuperioDevice *self, FuChunk *chk, GError
 						error);
 	if (fw3 == NULL) {
 		g_prefix_error (error, "failed to read written "
-				"bytes @0x%04x", (guint) chk->address);
+				"bytes @0x%04x: ", (guint) chk->address);
 		return FALSE;
 	}
 	if (!fu_common_bytes_compare (fw2, fw3, error)) {
-		g_prefix_error (error, "failed to verify @0x%04x",
+		g_prefix_error (error, "failed to verify @0x%04x: ",
 				(guint) chk->address);
 		return FALSE;
 	}
@@ -681,6 +698,7 @@ fu_superio_it89_device_class_init (FuSuperioIt89DeviceClass *klass)
 	klass_device->attach = fu_superio_it89_device_attach;
 	klass_device->detach = fu_superio_it89_device_detach;
 	klass_device->read_firmware = fu_superio_it89_device_read_firmware;
+	klass_device->dump_firmware = fu_superio_it89_device_dump_firmware;
 	klass_device->write_firmware = fu_superio_it89_device_write_firmware;
 	klass_superio_device->setup = fu_superio_it89_device_setup;
 }
