@@ -67,7 +67,7 @@ fu_synaptics_cxaudio_device_output_report (FuSynapticsCxaudioDevice *self,
 	return fu_hid_device_set_report (FU_HID_DEVICE (self), buf[0],
 					 buf, bufsz,
 					 FU_SYNAPTICS_CXAUDIO_USB_TIMEOUT,
-					 FU_HID_DEVICE_FLAG_NONE,
+					 FU_HID_DEVICE_FLAG_RETRY_FAILURE,
 					 error);
 }
 
@@ -550,8 +550,6 @@ fu_synaptics_cxaudio_device_prepare_firmware (FuDevice *device,
 	FuSynapticsCxaudioDevice *self = FU_SYNAPTICS_CXAUDIO_DEVICE (device);
 	guint32 chip_id_base;
 	g_autoptr(FuFirmware) firmware = fu_synaptics_cxaudio_firmware_new ();
-
-	fu_device_set_status (device, FWUPD_STATUS_DECOMPRESSING);
 	if (!fu_firmware_parse (firmware, fw, flags, error))
 		return NULL;
 	chip_id_base = fu_synaptics_cxaudio_firmware_get_devtype (FU_SYNAPTICS_CXAUDIO_FIRMWARE (firmware));
@@ -612,7 +610,7 @@ fu_synaptics_cxaudio_device_write_firmware (FuDevice *device,
 							    &value, sizeof(value),
 							    FU_SYNAPTICS_CXAUDIO_OPERATION_FLAG_NONE,
 							    error)) {
-			g_prefix_error (error, "failed to initialize layout signature ");
+			g_prefix_error (error, "failed to initialize layout signature: ");
 			return FALSE;
 		}
 		if (!fu_synaptics_cxaudio_device_operation (self,
@@ -622,7 +620,7 @@ fu_synaptics_cxaudio_device_write_firmware (FuDevice *device,
 							    &value, sizeof(value),
 							    FU_SYNAPTICS_CXAUDIO_OPERATION_FLAG_NONE,
 							    error)) {
-			g_prefix_error (error, "failed to initialize layout signature ");
+			g_prefix_error (error, "failed to initialize layout signature: ");
 			return FALSE;
 		}
 		g_debug ("initialized layout signature");
@@ -672,7 +670,7 @@ fu_synaptics_cxaudio_device_write_firmware (FuDevice *device,
 								    (guint8 *) &pinfo, sizeof(pinfo),
 								    FU_SYNAPTICS_CXAUDIO_OPERATION_FLAG_NONE,
 								    error)) {
-				g_prefix_error (error, "failed to write empty EEPROM patch info");
+				g_prefix_error (error, "failed to write empty EEPROM patch info: ");
 				return FALSE;
 			}
 			g_debug ("invalidated old FW patch for CX2070x (RAM) device");
@@ -762,6 +760,7 @@ fu_synaptics_cxaudio_device_init (FuSynapticsCxaudioDevice *self)
 	fu_device_set_version_format (FU_DEVICE (self), FWUPD_VERSION_FORMAT_PLAIN);
 	fu_device_set_install_duration (FU_DEVICE (self), 3); /* seconds */
 	fu_device_set_protocol (FU_DEVICE (self), "com.synaptics.cxaudio");
+	fu_device_retry_set_delay (FU_DEVICE (self), 100); /* ms */
 	fu_device_set_remove_delay (FU_DEVICE (self), FU_DEVICE_REMOVE_DELAY_RE_ENUMERATE);
 }
 
