@@ -20,10 +20,17 @@
 #include "fu-thunderbolt-device.h"
 #include "fu-thunderbolt-firmware.h"
 #include "fu-thunderbolt-firmware-update.h"
+#include "fu-udev-device-private.h"
+
+typedef enum {
+	FU_THUNDERBOLT_DEVICE_TYPE_DEVICE_CONTROLLER,
+	FU_THUNDERBOLT_DEVICE_TYPE_HOST_CONTROLLER,
+	FU_THUNDERBOLT_DEVICE_TYPE_RETIMER
+} FuThunderboltDeviceType;
 
 struct _FuThunderboltDevice {
 	FuUdevDevice		 parent_instance;
-	gboolean		 host;
+	FuThunderboltDeviceType	 device_type;
 	gboolean		 safe_mode;
 	gboolean		 is_native;
 	guint16			 gen;
@@ -103,6 +110,42 @@ fu_thunderbolt_device_read_status_block (FuThunderboltDevice *self, GError **err
 	return TRUE;
 }
 
+static gboolean
+fu_thunderbolt_device_check_authorized (FuThunderboltDevice *self, GError **error)
+{
+	guint64 status;
+	g_autofree gchar *attribute = NULL;
+	const gchar *update_error = NULL;
+	/* read directly from file to prevent udev caching */
+	g_autofree gchar *safe_path = g_build_path ("/", self->devpath, "authorized", NULL);
+
+	if (!g_file_test (safe_path, G_FILE_TEST_EXISTS)) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "missing authorized attribute");
+		return FALSE;
+	}
+
+	if (!g_file_get_contents (safe_path, &attribute, NULL, error))
+		return FALSE;
+	status = g_ascii_strtoull (attribute, NULL, 16);
+	if (status == G_MAXUINT64 && errno == ERANGE) {
+		g_set_error (error, G_IO_ERROR,
+			     g_io_error_from_errno (errno),
+			     "failed to read 'authorized: %s",
+			     g_strerror (errno));
+		return FALSE;
+	}
+	if (status == 1 || status == 2)
+		fu_device_add_flag (FU_DEVICE (self), FWUPD_DEVICE_FLAG_UPDATABLE);
+	else
+		update_error = "Not authorized";
+	fu_device_set_update_error (FU_DEVICE (self), update_error);
+
+	return TRUE;
+}
+
 static gboolean
 fu_thunderbolt_device_can_update (FuThunderboltDevice *self)
 {
@@ -120,7 +163,7 @@ fu_thunderbolt_device_can_update (FuThunderboltDevice *self)
 }
 
 static gboolean
-fu_thunderbolt_device_get_version (FuThunderboltDevice *self)
+fu_thunderbolt_device_get_version (FuThunderboltDevice *self, GError **error)
 {
 	g_auto(GStrv) split = NULL;
 	g_autofree gchar *version_raw = NULL;
@@ -128,6 +171,14 @@ fu_thunderbolt_device_get_version (FuThunderboltDevice *self)
 	/* read directly from file to prevent udev caching */
 	g_autofree gchar *safe_path = g_build_path ("/", self->devpath, "nvm_version", NULL);
 
+	if (!g_file_test (safe_path, G_FILE_TEST_EXISTS)) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "missing nvm_version attribute");
+		return FALSE;
+	}
+
 	for (guint i = 0; i < 50; i++) {
 		g_autoptr(GError) error_local = NULL;
 		/* glib can't return a properly mapped -ENODATA but the
@@ -141,18 +192,26 @@ fu_thunderbolt_device_get_version (FuThunderboltDevice *self)
 			break;
 	}
 
-	if (version_raw == NULL)
+	if (version_raw == NULL) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_INTERNAL,
+				     "failed to read NVM");
 		return FALSE;
+	}
 	split = g_strsplit (version_raw, ".", -1);
-	if (g_strv_length (split) != 2)
+	if (g_strv_length (split) != 2) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "invalid nvm_version format: %s", version_raw);
 		return FALSE;
+	}
 
 	version = g_strdup_printf ("%02x.%02x",
 				   (guint) g_ascii_strtoull (split[0], NULL, 16),
 				   (guint) g_ascii_strtoull (split[1], NULL, 16));
 	fu_device_set_version (FU_DEVICE (self), version);
-	g_debug ("setting version to %s", version);
-	g_debug ("path is %s", self->devpath);
 	return TRUE;
 }
 
@@ -160,7 +219,7 @@ static void
 fu_thunderbolt_device_check_safe_mode (FuThunderboltDevice *self)
 {
 	/* failed to read, for host check for safe mode */
-	if (!self->host || self->gen >= 4)
+	if (self->device_type != FU_THUNDERBOLT_DEVICE_TYPE_DEVICE_CONTROLLER)
 		return;
 	g_warning ("%s is in safe mode --  VID/DID will "
 		   "need to be set by another plugin",
@@ -171,11 +230,31 @@ fu_thunderbolt_device_check_safe_mode (FuThunderboltDevice *self)
 	fu_device_set_metadata_boolean (FU_DEVICE (self), FU_DEVICE_METADATA_TBT_IS_SAFE_MODE, TRUE);
 }
 
+static const gchar*
+fu_thunderbolt_device_type_to_string (FuThunderboltDevice *self)
+{
+	if (self->device_type == FU_THUNDERBOLT_DEVICE_TYPE_HOST_CONTROLLER) {
+		if (self->gen >= 4)
+			return "USB4 host controller";
+		else
+			return "Thunderbolt host controller";
+	}
+	if (self->device_type == FU_THUNDERBOLT_DEVICE_TYPE_DEVICE_CONTROLLER) {
+		if (self->gen >= 4)
+			return "USB4 device controller";
+		else
+			return "Thunderbolt device controller";
+	}
+	if (self->device_type == FU_THUNDERBOLT_DEVICE_TYPE_RETIMER)
+		return "USB4 Retimer";
+	return "Unknown";
+}
+
 static void
 fu_thunderbolt_device_to_string (FuUdevDevice *device, guint idt, GString *str)
 {
 	FuThunderboltDevice *self = FU_THUNDERBOLT_DEVICE (device);
-	fu_common_string_append_kb (str, idt, "Host Controller", self->host);
+	fu_common_string_append_kv (str, idt, "Device Type", fu_thunderbolt_device_type_to_string (self));
 	fu_common_string_append_kb (str, idt, "Safe Mode", self->safe_mode);
 	fu_common_string_append_kb (str, idt, "Native mode", self->is_native);
 	fu_common_string_append_ku (str, idt, "Generation", self->gen);
@@ -185,16 +264,28 @@ fu_thunderbolt_device_to_string (FuUdevDevice *device, guint idt, GString *str)
 static gboolean
 fu_thunderbolt_device_probe (FuUdevDevice *device, GError **error)
 {
-	const gchar *tmp = fu_udev_device_get_sysfs_attr (device, "unique_id", NULL);
-	/* most likely the domain itself, ignore */
-	if (tmp == NULL) {
-		g_set_error_literal (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_NOT_SUPPORTED,
-				     "thunderbolt domain not used");
+	FuThunderboltDevice *self = FU_THUNDERBOLT_DEVICE (device);
+	const gchar *tmp = fu_udev_device_get_devtype (device);
+
+	/* device */
+	if (g_strcmp0 (tmp, "thunderbolt_device") == 0) {
+		tmp = fu_udev_device_get_sysfs_attr (device, "unique_id", NULL);
+		if (tmp != NULL)
+			fu_device_set_physical_id (FU_DEVICE (device), tmp);
+	/* retimer */
+	} else if (g_strcmp0 (tmp, "thunderbolt_retimer") == 0) {
+		self->device_type = FU_THUNDERBOLT_DEVICE_TYPE_RETIMER;
+		tmp = g_path_get_basename (fu_udev_device_get_sysfs_path (FU_UDEV_DEVICE (device)));
+		if (tmp != NULL)
+			fu_device_set_physical_id (FU_DEVICE (device), tmp);
+	/* domain or unsupported */
+	} else {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_SUPPORTED,
+			     "%s not used", tmp);
 		return FALSE;
 	}
-	fu_device_set_physical_id (FU_DEVICE (device), tmp);
 
 	return TRUE;
 }
@@ -231,7 +322,7 @@ fu_thunderbolt_device_get_attr_uint16 (FuThunderboltDevice *self,
 }
 
 static gboolean
-fu_thunderbolt_device_setup (FuDevice *device, GError **error)
+fu_thunderbolt_device_setup_controller (FuDevice *device, GError **error)
 {
 	FuThunderboltDevice *self = FU_THUNDERBOLT_DEVICE (device);
 	const gchar *tmp = NULL;
@@ -240,9 +331,6 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 	g_autoptr(GError) error_gen = NULL;
 	g_autofree gchar *parent_name = fu_udev_device_get_parent_name (FU_UDEV_DEVICE (self));
 
-	self->devpath = g_strdup (fu_udev_device_get_sysfs_path (FU_UDEV_DEVICE (device)));
-	fu_device_set_metadata (device, "sysfs-path", self->devpath);
-
 	/* these may be missing on ICL or later */
 	vid = fu_udev_device_get_vendor (FU_UDEV_DEVICE (self));
 	if (vid == 0x0)
@@ -263,7 +351,7 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 
 	/* determine if host controller or not */
 	if (parent_name != NULL && g_str_has_prefix (parent_name, "domain")) {
-		self->host = TRUE;
+		self->device_type = FU_THUNDERBOLT_DEVICE_TYPE_HOST_CONTROLLER;
 		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_INTERNAL);
 		fu_device_set_summary (device, "Unmatched performance for high-speed I/O");
 	} else {
@@ -271,12 +359,8 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 	}
 
 	/* set the controller name */
-	if (tmp == NULL) {
-		if (self->gen == 4)
-			tmp = "USB4 Controller";
-		else
-			tmp = "Thunderbolt Controller";
-	}
+	if (tmp == NULL)
+		tmp = fu_thunderbolt_device_type_to_string (self);
 	fu_device_set_name (device, tmp);
 
 	/* set vendor string */
@@ -285,8 +369,7 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 		return FALSE;
 	fu_device_set_vendor (device, tmp);
 
-	/* try to read the version */
-	if (!fu_thunderbolt_device_get_version (self))
+	if (fu_device_get_version (device) == NULL)
 		fu_thunderbolt_device_check_safe_mode (self);
 
 	if (self->safe_mode) {
@@ -299,7 +382,8 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 			g_autofree gchar *domain = g_path_get_basename (self->devpath);
 			/* USB4 controllers don't have a concept of legacy vs native
 			 * so don't try to read a native attribute from their NVM */
-			if (self->host && self->gen < 4) {
+			if (self->device_type == FU_THUNDERBOLT_DEVICE_TYPE_HOST_CONTROLLER
+			    && self->gen < 4) {
 				domain_id = g_strdup_printf ("TBT-%04x%04x%s-controller%s",
 							     (guint) vid,
 							     (guint) did,
@@ -312,8 +396,12 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 						     (guint) vid,
 						     (guint) did,
 						     self->is_native ? "-native" : "");
-			fu_device_add_flag (device, FWUPD_DEVICE_FLAG_UPDATABLE);
 			fu_device_add_flag (device, FWUPD_DEVICE_FLAG_DUAL_IMAGE);
+
+			/* check if device is authorized */
+			if (!fu_thunderbolt_device_check_authorized (self, error))
+				return FALSE;
+
 		} else {
 			device_id = g_strdup ("TBT-fixed");
 		}
@@ -331,14 +419,97 @@ fu_thunderbolt_device_setup (FuDevice *device, GError **error)
 		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE);
 		/* forces the device to write to authenticate on disconnect attribute */
 		fu_device_remove_flag (device, FWUPD_DEVICE_FLAG_SKIPS_RESTART);
-	} else {
-		self->auth_method = "nvm_authenticate";
+		/* control the order of activation (less relevant; install too though) */
+		fu_device_add_flag (device, FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST);
 	}
 
-	/* success */
 	return TRUE;
 }
 
+static gboolean
+fu_thunderbolt_device_setup_retimer (FuDevice *device, GError **error)
+{
+	FuThunderboltDevice *self = FU_THUNDERBOLT_DEVICE (device);
+	guint16 did;
+	guint16 vid;
+	g_autofree gchar *instance = NULL;
+
+	/* as defined in PCIe 4.0 spec */
+	fu_device_set_summary (device, "A physical layer protocol-aware, software-transparent extension device "
+				        "that forms two separate electrical link segments");
+	fu_device_set_name (device, fu_thunderbolt_device_type_to_string (self));
+	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_UPDATABLE);
+	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_DUAL_IMAGE);
+	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_INTERNAL);
+	vid = fu_udev_device_get_vendor (FU_UDEV_DEVICE (self));
+	if (vid == 0x0) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "missing vendor id");
+		return FALSE;
+	}
+
+	did = fu_udev_device_get_model (FU_UDEV_DEVICE (self));
+	if (did == 0x0) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "missing device id");
+		return FALSE;
+
+	}
+
+	instance = g_strdup_printf ("TBT-%04x%04x-retimer%s",
+				    (guint) vid,
+				    (guint) did,
+				    fu_device_get_physical_id (device));
+	fu_device_add_instance_id (device, instance);
+
+	/* hardcoded for now:
+	 * 1. unsure if ID_VENDOR_FROM_DATABASE works in this instance
+	 * 2. we don't recognize anyone else yet
+	 */
+	if (fu_device_get_vendor (device) == NULL)
+		fu_device_set_vendor (device, "Intel");
+
+	return TRUE;
+}
+
+static gboolean
+fu_thunderbolt_device_setup (FuDevice *device, GError **error)
+{
+	FuThunderboltDevice *self = FU_THUNDERBOLT_DEVICE (device);
+	g_autoptr(GError) error_version = NULL;
+
+	self->devpath = g_strdup (fu_udev_device_get_sysfs_path (FU_UDEV_DEVICE (device)));
+
+	/* try to read the version */
+	if (!fu_thunderbolt_device_get_version (self, &error_version)) {
+		if (g_error_matches (error_version, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED)) {
+			g_propagate_error (error, g_steal_pointer (&error_version));
+			return FALSE;
+		}
+		g_debug ("%s", error_version->message);
+	}
+
+	/* default behavior */
+	self->auth_method = "nvm_authenticate";
+
+	/* configure differences between retimer and controller */
+	if (self->device_type == FU_THUNDERBOLT_DEVICE_TYPE_RETIMER)
+		return fu_thunderbolt_device_setup_retimer (device, error);
+	return fu_thunderbolt_device_setup_controller (device, error);
+}
+
+static gboolean
+fu_thunderbolt_device_activate (FuDevice *device, GError **error)
+{
+	FuUdevDevice *udev = FU_UDEV_DEVICE (device);
+
+	return fu_udev_device_write_sysfs (udev, "nvm_authenticate", "1", error);
+}
+
 static gboolean
 fu_thunderbolt_device_authenticate (FuDevice *device, GError **error)
 {
@@ -394,8 +565,13 @@ static gboolean
 fu_thunderbolt_device_rescan (FuDevice *device, GError **error)
 {
 	FuThunderboltDevice *self = FU_THUNDERBOLT_DEVICE (device);
+
+	/* refresh updatability */
+	if (!fu_thunderbolt_device_check_authorized (self, error))
+		return FALSE;
+
 	/* refresh the version */
-	return fu_thunderbolt_device_get_version (self);
+	return fu_thunderbolt_device_get_version (self, error);
 }
 
 static gboolean
@@ -476,8 +652,6 @@ fu_thunderbolt_device_prepare_firmware (FuDevice *device,
 	if (nvmem == NULL)
 		return NULL;
 	controller_fw = g_file_load_bytes (nvmem, NULL, NULL, error);
-	if (controller_fw == NULL)
-		return NULL;
 	if (!fu_firmware_parse (FU_FIRMWARE (firmware_old), controller_fw, flags, error))
 		return NULL;
 	if (fu_thunderbolt_firmware_is_host (FU_THUNDERBOLT_FIRMWARE (firmware)) !=
@@ -510,7 +684,7 @@ fu_thunderbolt_device_prepare_firmware (FuDevice *device,
 			     fu_thunderbolt_firmware_get_device_id (firmware_old));
 		return NULL;
 	}
-	if ((flags & FWUPD_INSTALL_FLAG_FORCE) == 0) {
+	if ((flags & FWUPD_INSTALL_FLAG_IGNORE_VID_PID) == 0) {
 		if (fu_thunderbolt_firmware_get_model_id (FU_THUNDERBOLT_FIRMWARE (firmware)) !=
 		    fu_thunderbolt_firmware_get_model_id (firmware_old)) {
 			g_set_error (error,
@@ -620,7 +794,7 @@ fu_thunderbolt_device_class_init (FuThunderboltDeviceClass *klass)
 	FuDeviceClass *klass_device = FU_DEVICE_CLASS (klass);
 	FuUdevDeviceClass *klass_udev_device = FU_UDEV_DEVICE_CLASS (klass);
 	object_class->finalize = fu_thunderbolt_device_finalize;
-	klass_device->activate = fu_thunderbolt_device_authenticate;
+	klass_device->activate = fu_thunderbolt_device_activate;
 	klass_udev_device->to_string = fu_thunderbolt_device_to_string;
 	klass_device->setup = fu_thunderbolt_device_setup;
 	klass_device->prepare_firmware = fu_thunderbolt_device_prepare_firmware;
