@@ -38,8 +38,9 @@ fu_plugin_init (FuPlugin *plugin)
 	FuPluginData *data = fu_plugin_alloc_data (plugin, sizeof (FuPluginData));
 	data->bgrt = fu_uefi_bgrt_new ();
 	fu_plugin_add_rule (plugin, FU_PLUGIN_RULE_RUN_AFTER, "upower");
+	fu_plugin_add_rule (plugin, FU_PLUGIN_RULE_METADATA_SOURCE, "tpm");
 	fu_plugin_add_rule (plugin, FU_PLUGIN_RULE_METADATA_SOURCE, "tpm_eventlog");
-	fu_plugin_add_compile_version (plugin, "com.redhat.efivar", EFIVAR_LIBRARY_VERSION);
+	fu_plugin_add_rule (plugin, FU_PLUGIN_RULE_METADATA_SOURCE, "dell");
 	fu_plugin_set_build_hash (plugin, FU_BUILD_HASH);
 }
 
@@ -94,6 +95,35 @@ fu_plugin_get_results (FuPlugin *plugin, FuDevice *device, GError **error)
 	return TRUE;
 }
 
+void
+fu_plugin_add_security_attrs (FuPlugin *plugin, FuSecurityAttrs *attrs)
+{
+	g_autoptr(FwupdSecurityAttr) attr = NULL;
+	g_autoptr(GError) error = NULL;
+
+	/* create attr */
+	attr = fwupd_security_attr_new (FWUPD_SECURITY_ATTR_ID_UEFI_SECUREBOOT);
+	fwupd_security_attr_set_plugin (attr, fu_plugin_get_name (plugin));
+	fu_security_attrs_append (attrs, attr);
+
+	/* SB not available or disabled */
+	if (!fu_efivar_secure_boot_enabled_full (&error)) {
+		if (g_error_matches (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED)) {
+			fwupd_security_attr_set_result (attr, FWUPD_SECURITY_ATTR_RESULT_NOT_FOUND);
+			return;
+		}
+		fwupd_security_attr_add_flag (attr, FWUPD_SECURITY_ATTR_FLAG_RUNTIME_ISSUE);
+		fwupd_security_attr_set_result (attr, FWUPD_SECURITY_ATTR_RESULT_NOT_ENABLED);
+		return;
+	}
+
+	/* success */
+	fwupd_security_attr_add_flag (attr, FWUPD_SECURITY_ATTR_FLAG_SUCCESS);
+	fwupd_security_attr_set_result (attr, FWUPD_SECURITY_ATTR_RESULT_ENABLED);
+}
+
 static GBytes *
 fu_plugin_uefi_get_splash_data (guint width, guint height, GError **error)
 {
@@ -189,7 +219,7 @@ fu_plugin_uefi_write_splash_data (FuPlugin *plugin,
 	efi_ux_capsule_header_t header = { 0 };
 	efi_capsule_header_t capsule_header = {
 		.flags = EFI_CAPSULE_HEADER_FLAGS_PERSIST_ACROSS_RESET,
-		.guid = efi_guid_ux_capsule,
+		.guid = { 0x0 },
 		.header_size = sizeof(efi_capsule_header_t),
 		.capsule_image_size = 0
 	};
@@ -211,7 +241,7 @@ fu_plugin_uefi_write_splash_data (FuPlugin *plugin,
 
 	/* save to a predicatable filename */
 	esp_path = fu_volume_get_mount_point (data->esp);
-	directory = fu_uefi_get_esp_path_for_os (esp_path);
+	directory = fu_uefi_get_esp_path_for_os (device, esp_path);
 	basename = g_strdup_printf ("fwupd-%s.cap", FU_EFIVAR_GUID_UX_CAPSULE);
 	fn = g_build_filename (directory, "fw", basename, NULL);
 	if (!fu_common_mkdir_parent (fn, error))
@@ -222,6 +252,11 @@ fu_plugin_uefi_write_splash_data (FuPlugin *plugin,
 	if (ostream == NULL)
 		return FALSE;
 
+	if (!fwupd_guid_from_string (FU_EFIVAR_GUID_UX_CAPSULE,
+				     &capsule_header.guid,
+				     FWUPD_GUID_FLAG_MIXED_ENDIAN,
+				     error))
+		return FALSE;
 	capsule_header.capsule_image_size =
 		g_bytes_get_size (blob) +
 		sizeof(efi_capsule_header_t) +
@@ -256,13 +291,10 @@ fu_plugin_uefi_write_splash_data (FuPlugin *plugin,
 		return FALSE;
 
 	/* write display capsule location as UPDATE_INFO */
-	if (!fu_uefi_device_write_update_info (FU_UEFI_DEVICE (device), fn,
-					       "fwupd-ux-capsule",
-					       &efi_guid_ux_capsule, error))
-		return FALSE;
-
-	/* success */
-	return TRUE;
+	return fu_uefi_device_write_update_info (FU_UEFI_DEVICE (device), fn,
+						 "fwupd-ux-capsule",
+						 FU_EFIVAR_GUID_UX_CAPSULE,
+						 error);
 }
 
 static gboolean
@@ -380,31 +412,20 @@ fu_plugin_update (FuPlugin *plugin,
 	g_assert (str != NULL);
 
 	/* perform the update */
-	g_debug ("Performing UEFI capsule update");
 	fu_device_set_status (device, FWUPD_STATUS_SCHEDULING);
 	if (!fu_plugin_uefi_update_splash (plugin, device, &error_splash)) {
 		g_debug ("failed to upload UEFI UX capsule text: %s",
 			 error_splash->message);
 	}
 
-	if (!fu_device_write_firmware (device, blob_fw, flags, error))
-		return FALSE;
-
-	/* record if we had an invalid header during update */
-	str = fu_uefi_missing_capsule_header (device) ? "True" : "False";
-	fu_plugin_add_report_metadata (plugin, "MissingCapsuleHeader", str);
-
-	/* where the ESP was mounted during installation */
-	str = fu_device_get_metadata (device, "EspPath");
-	fu_plugin_add_report_metadata (plugin, "ESPMountPoint", str);
-
-	return TRUE;
+	return fu_device_write_firmware (device, blob_fw, flags, error);
 }
 
 static void
 fu_plugin_uefi_load_config (FuPlugin *plugin, FuDevice *device)
 {
 	gboolean disable_shim;
+	gboolean fallback_removable_path;
 	guint64 sz_reqd = FU_UEFI_COMMON_REQUIRED_ESP_FREE_SPACE;
 	g_autofree gchar *require_esp_free_space = NULL;
 
@@ -419,6 +440,12 @@ fu_plugin_uefi_load_config (FuPlugin *plugin, FuDevice *device)
 	fu_device_set_metadata_boolean (device,
 					"RequireShimForSecureBoot",
 					!disable_shim);
+
+	/* check if using UEFI removeable path */
+	fallback_removable_path = fu_plugin_get_config_value_boolean (plugin, "FallbacktoRemovablePath");
+	fu_device_set_metadata_boolean (device,
+					"FallbacktoRemovablePath",
+					fallback_removable_path);
 }
 
 static void
@@ -490,6 +517,8 @@ fu_plugin_uefi_coldplug_device (FuPlugin *plugin, FuUefiDevice *dev, GError **er
 	/* probe to get add GUIDs (and hence any quirk fixups) */
 	if (!fu_device_probe (FU_DEVICE (dev), error))
 		return FALSE;
+	if (!fu_device_setup (FU_DEVICE (dev), error))
+		return FALSE;
 
 	/* if not already set by quirks */
 	if (fu_device_get_custom_flags (FU_DEVICE (dev)) == NULL) {
@@ -586,9 +615,15 @@ gboolean
 fu_plugin_startup (FuPlugin *plugin, GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
+	guint64 nvram_total;
 	g_autofree gchar *esp_path = NULL;
+	g_autofree gchar *nvram_total_str = NULL;
 	g_autoptr(GError) error_local = NULL;
 
+	/* don't let user's environment influence test suite failures */
+	if (g_getenv ("FWUPD_UEFI_TEST") != NULL)
+		return TRUE;
+
 	/* some platforms have broken SMBIOS data */
 	if (fu_plugin_has_custom_flag (plugin, "uefi-force-enable"))
 		return TRUE;
@@ -610,6 +645,11 @@ fu_plugin_startup (FuPlugin *plugin, GError **error)
 	/* are the EFI dirs set up so we can update each device */
 	if (!fu_efivar_supported (error))
 		return FALSE;
+	nvram_total = fu_efivar_space_used (error);
+	if (nvram_total == G_MAXUINT64)
+		return FALSE;
+	nvram_total_str = g_format_size_full (nvram_total, G_FORMAT_SIZE_LONG_FORMAT);
+	fu_plugin_add_report_metadata (plugin, "EfivarNvramUsed", nvram_total_str);
 
 	/* override the default ESP path */
 	esp_path = fu_plugin_get_config_value (plugin, "OverrideESPMountPoint");
@@ -711,7 +751,7 @@ fu_plugin_unlock (FuPlugin *plugin, FuDevice *device, GError **error)
 	/* clone the info from real device but prevent it from being flashed */
 	device_flags_alt = fu_device_get_flags (device_alt);
 	fu_device_set_flags (device, device_flags_alt);
-	fu_device_remove_flag (device_alt, FWUPD_DEVICE_FLAG_UPDATABLE);
+	fu_device_set_flags (device_alt, device_flags_alt & ~FWUPD_DEVICE_FLAG_UPDATABLE);
 
 	/* make sure that this unlocked device can be updated */
 	fu_device_set_version_format (device, FWUPD_VERSION_FORMAT_QUAD);
@@ -719,43 +759,6 @@ fu_plugin_unlock (FuPlugin *plugin, FuDevice *device, GError **error)
 	return TRUE;
 }
 
-static gboolean
-fu_plugin_uefi_create_dummy (FuPlugin *plugin, const gchar *reason, GError **error)
-{
-	const gchar *key;
-	g_autoptr(FuDevice) dev = fu_device_new ();
-
-	fu_device_set_version_format (dev, FWUPD_VERSION_FORMAT_PLAIN);
-	key = fu_plugin_get_dmi_value (plugin, FU_HWIDS_KEY_MANUFACTURER);
-	if (key != NULL)
-		fu_device_set_vendor (dev, key);
-	key = fu_plugin_get_dmi_value (plugin, FU_HWIDS_KEY_BIOS_VENDOR);
-	if (key != NULL) {
-		g_autofree gchar *vendor_id = g_strdup_printf ("DMI:%s", key);
-		fu_device_set_vendor_id (FU_DEVICE (dev), vendor_id);
-	}
-	key = fu_plugin_uefi_get_name_for_type (plugin, FU_UEFI_DEVICE_KIND_SYSTEM_FIRMWARE);
-	fu_device_set_name (dev, key);
-	key = fu_plugin_get_dmi_value (plugin, FU_HWIDS_KEY_BIOS_VERSION);
-	if (key != NULL)
-		fu_device_set_version (dev, key);
-	fu_device_set_update_error (dev, reason);
-
-	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_INTERNAL);
-	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_NEEDS_REBOOT);
-	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_REQUIRE_AC);
-	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_MD_SET_VERFMT);
-
-	fu_device_add_icon (dev, "computer");
-	fu_device_set_id (dev, "UEFI-dummy");
-	fu_device_add_instance_id (dev, "main-system-firmware");
-	if (!fu_device_setup (dev, error))
-		return FALSE;
-	fu_plugin_device_add (plugin, dev);
-
-	return TRUE;
-}
-
 gboolean
 fu_plugin_coldplug (FuPlugin *plugin, GError **error)
 {
@@ -768,31 +771,29 @@ fu_plugin_coldplug (FuPlugin *plugin, GError **error)
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) entries = NULL;
 
-	/* are the EFI dirs set up so we can update each device */
-	if (!fu_efivar_supported (&error_local)) {
-		const gchar *reason = "Firmware can not be updated in legacy mode, switch to UEFI mode";
-		g_warning ("%s", error_local->message);
-		return fu_plugin_uefi_create_dummy (plugin, reason, error);
-	}
-
 	/* get the directory of ESRT entries */
 	sysfsfwdir = fu_common_get_path (FU_PATH_KIND_SYSFSDIR_FW);
 	esrt_path = g_build_filename (sysfsfwdir, "efi", "esrt", NULL);
-	entries = fu_uefi_get_esrt_entry_paths (esrt_path, &error_local);
-	if (entries == NULL) {
-		const gchar *reason = "UEFI Capsule updates not available or enabled";
-		g_warning ("%s", error_local->message);
-		return fu_plugin_uefi_create_dummy (plugin, reason, error);
-	}
+	entries = fu_uefi_get_esrt_entry_paths (esrt_path, error);
+	if (entries == NULL)
+		return FALSE;
 
 	/* make sure that efivarfs is rw */
-	if (!fu_plugin_uefi_ensure_efivarfs_rw (&error_efivarfs))
+	if (!fu_plugin_uefi_ensure_efivarfs_rw (&error_efivarfs)) {
+		fu_plugin_add_flag (plugin, FWUPD_PLUGIN_FLAG_EFIVAR_NOT_MOUNTED);
+		fu_plugin_add_flag (plugin, FWUPD_PLUGIN_FLAG_CLEAR_UPDATABLE);
+		fu_plugin_add_flag (plugin, FWUPD_PLUGIN_FLAG_USER_WARNING);
 		g_warning ("%s", error_efivarfs->message);
+	}
 
 	if (data->esp == NULL) {
 		data->esp = fu_common_get_esp_default (&error_udisks2);
-		if (data->esp == NULL)
+		if (data->esp == NULL) {
+			fu_plugin_add_flag (plugin, FWUPD_PLUGIN_FLAG_ESP_NOT_FOUND);
+			fu_plugin_add_flag (plugin, FWUPD_PLUGIN_FLAG_CLEAR_UPDATABLE);
+			fu_plugin_add_flag (plugin, FWUPD_PLUGIN_FLAG_USER_WARNING);
 			g_warning ("cannot find default ESP: %s", error_udisks2->message);
+		}
 	}
 
 	/* add each device */
@@ -809,14 +810,9 @@ fu_plugin_coldplug (FuPlugin *plugin, GError **error)
 			fu_uefi_device_set_esp (FU_UEFI_DEVICE (dev), data->esp);
 		if (!fu_plugin_uefi_coldplug_device (plugin, dev, error))
 			return FALSE;
-		if (error_efivarfs != NULL) {
-			fu_device_set_update_error (FU_DEVICE (dev), error_efivarfs->message);
-		} else if (error_udisks2 != NULL) {
-			fu_device_set_update_error (FU_DEVICE (dev), error_udisks2->message);
-		} else {
-			fu_device_add_flag (FU_DEVICE (dev), FWUPD_DEVICE_FLAG_UPDATABLE);
-			fu_device_add_flag (FU_DEVICE (dev), FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE);
-		}
+		fu_device_add_flag (FU_DEVICE (dev), FWUPD_DEVICE_FLAG_UPDATABLE);
+		fu_device_add_flag (FU_DEVICE (dev), FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE);
+
 		/* load all configuration variables */
 		fu_plugin_uefi_load_config (plugin, FU_DEVICE (dev));
 		fu_plugin_device_add (plugin, FU_DEVICE (dev));
