@@ -8,7 +8,10 @@
 #pragma once
 
 #include <glib.h>
-#include <efivar.h>
+
+#include "fwupd-common.h"
+
+#include "fu-device.h"
 
 #define EFI_CAPSULE_HEADER_FLAGS_PERSIST_ACROSS_RESET	0x00010000
 #define EFI_CAPSULE_HEADER_FLAGS_POPULATE_SYSTEM_TABLE	0x00020000
@@ -29,7 +32,7 @@ typedef struct __attribute__((__packed__)) {
 } efi_time_t;
 
 typedef struct __attribute__((__packed__)) {
-	efi_guid_t	 guid;
+	fwupd_guid_t	 guid;
 	guint32		 header_size;
 	guint32		 flags;
 	guint32		 capsule_image_size;
@@ -47,7 +50,7 @@ typedef struct __attribute__((__packed__)) {
 
 typedef struct __attribute__((__packed__)) {
 	guint32		 update_info_version;
-	efi_guid_t	 guid;
+	fwupd_guid_t	 guid;
 	guint32		 capsule_flags;
 	guint64		 hw_inst;
 	efi_time_t	 time_attempted;
@@ -57,7 +60,8 @@ typedef struct __attribute__((__packed__)) {
 /* the biggest size SPI part currently seen */
 #define FU_UEFI_COMMON_REQUIRED_ESP_FREE_SPACE		(32 * 1024 * 1024)
 
-gchar		*fu_uefi_get_esp_app_path	(const gchar	*esp_path,
+gchar		*fu_uefi_get_esp_app_path	(FuDevice       *device,
+						 const gchar	*esp_path,
 						 const gchar	*cmd,
 						 GError		**error);
 gchar		*fu_uefi_get_built_app_path	(GError		**error);
@@ -69,9 +73,9 @@ gboolean	 fu_uefi_get_bitmap_size	(const guint8	*buf,
 gboolean	 fu_uefi_get_framebuffer_size	(guint32	*width,
 						 guint32	*height,
 						 GError		**error);
-gchar		*fu_uefi_get_esp_path_for_os	(const gchar	*esp_path);
+gchar		*fu_uefi_get_esp_path_for_os	(FuDevice 	*device,
+						 const gchar	*esp_path);
 GPtrArray	*fu_uefi_get_esrt_entry_paths	(const gchar	*esrt_path,
 						 GError		**error);
 guint64		 fu_uefi_read_file_as_uint64	(const gchar	*path,
 						 const gchar	*attr_name);
-void		 fu_uefi_print_efivar_errors	(void);
