@@ -43,8 +43,3 @@ const gchar	*fu_vli_common_device_kind_to_string	(FuVliDeviceKind	 device_kind);
 FuVliDeviceKind	 fu_vli_common_device_kind_from_string	(const gchar		*device_kind);
 guint32		 fu_vli_common_device_kind_get_size	(FuVliDeviceKind	 device_kind);
 guint32		 fu_vli_common_device_kind_get_offset	(FuVliDeviceKind	 device_kind);
-
-guint8		 fu_vli_common_crc8			(const guint8		*buf,
-							 gsize			 bufsz);
-guint16		 fu_vli_common_crc16			(const guint8		*buf,
-							 gsize			 bufsz);
