@@ -17,4 +17,7 @@ typedef struct __attribute__ ((packed)) {
 	guint16		pid;	/* LE */
 } FuVliPdHdr;
 
+#define VLI_USBHUB_PD_FLASHMAP_ADDR_LEGACY		0x4000
+#define VLI_USBHUB_PD_FLASHMAP_ADDR			0x1003
+
 FuVliDeviceKind	 fu_vli_pd_common_guess_device_kind		(guint32		 fwver);
