@@ -579,8 +579,8 @@ fu_vli_pd_parade_device_write_firmware (FuDevice *device,
 	return TRUE;
 }
 
-static FuFirmware *
-fu_vli_pd_parade_device_read_firmware (FuDevice *device, GError **error)
+static GBytes *
+fu_vli_pd_parade_device_dump_firmware (FuDevice *device, GError **error)
 {
 	FuVliPdDevice *parent = FU_VLI_PD_DEVICE (fu_device_get_parent (device));
 	FuVliPdParadeDevice *self = FU_VLI_PD_PARADE_DEVICE (device);
@@ -610,27 +610,7 @@ fu_vli_pd_parade_device_read_firmware (FuDevice *device, GError **error)
 							 error))
 			return NULL;
 	}
-	return fu_firmware_new_from_bytes (fw);
-}
-
-
-static FuFirmware *
-fu_vli_pd_parade_device_prepare_firmware (FuDevice *device,
-					  GBytes *fw,
-					  FwupdInstallFlags flags,
-					  GError **error)
-{
-	/* check size */
-	if (g_bytes_get_size (fw) < fu_device_get_firmware_size_min (device)) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_FILE,
-			     "firmware too small, got 0x%x, expected >= 0x%x",
-			     (guint) g_bytes_get_size (fw),
-			     (guint) fu_device_get_firmware_size_min (device));
-		return NULL;
-	}
-	return fu_firmware_new_from_bytes (fw);
+	return g_steal_pointer (&fw);
 }
 
 static gboolean
@@ -677,8 +657,7 @@ fu_vli_pd_parade_device_class_init (FuVliPdParadeDeviceClass *klass)
 	FuDeviceClass *klass_device = FU_DEVICE_CLASS (klass);
 	klass_device->to_string = fu_vli_pd_parade_device_to_string;
 	klass_device->probe = fu_vli_pd_parade_device_probe;
-	klass_device->read_firmware = fu_vli_pd_parade_device_read_firmware;
-	klass_device->prepare_firmware = fu_vli_pd_parade_device_prepare_firmware;
+	klass_device->dump_firmware = fu_vli_pd_parade_device_dump_firmware;
 	klass_device->write_firmware = fu_vli_pd_parade_device_write_firmware;
 }
 
