@@ -419,14 +419,16 @@ fu_vli_usbhub_device_guess_kind (FuVliUsbhubDevice *self, GError **error)
 		g_prefix_error (error, "Read_820Q7Q8 failed: ");
 		return FALSE;
 	}
-	g_debug ("chipver = 0x%02x", chipver);
-	g_debug ("chipver2 = 0x%02x", chipver2);
-	g_debug ("b811P812 = 0x%02x", b811P812);
-	g_debug ("chipid1 = 0x%02x", chipid1);
-	g_debug ("chipid2 = 0x%02x", chipid2);
-	g_debug ("chipid12 = 0x%02x", chipid12);
-	g_debug ("chipid22 = 0x%02x", chipid22);
-	g_debug ("b820Q7Q8 = 0x%02x", b820Q7Q8);
+	if (g_getenv ("FWUPD_VLI_USBHUB_VERBOSE") != NULL) {
+		g_debug ("chipver = 0x%02x", chipver);
+		g_debug ("chipver2 = 0x%02x", chipver2);
+		g_debug ("b811P812 = 0x%02x", b811P812);
+		g_debug ("chipid1 = 0x%02x", chipid1);
+		g_debug ("chipid2 = 0x%02x", chipid2);
+		g_debug ("chipid12 = 0x%02x", chipid12);
+		g_debug ("chipid22 = 0x%02x", chipid22);
+		g_debug ("b820Q7Q8 = 0x%02x", b820Q7Q8);
+	}
 
 	if (chipid2 == 0x35 && chipid1 == 0x07) {
 		fu_vli_device_set_kind (FU_VLI_DEVICE (self), FU_VLI_DEVICE_KIND_VL210);
@@ -496,43 +498,22 @@ fu_vli_usbhub_device_probe (FuDevice *device, GError **error)
 static gboolean
 fu_vli_usbhub_device_pd_setup (FuVliUsbhubDevice *self, GError **error)
 {
-	FuVliPdHdr hdr = { 0x0 };
 	g_autoptr(FuDevice) dev = NULL;
 	g_autoptr(GError) error_local = NULL;
 
-	/* legacy location */
-	if (!fu_vli_device_spi_read_block (FU_VLI_DEVICE (self),
-					   VLI_USBHUB_FLASHMAP_ADDR_PD_LEGACY +
-					   VLI_USBHUB_PD_FLASHMAP_ADDR_LEGACY,
-					   (guint8 *) &hdr, sizeof(hdr), error)) {
-		g_prefix_error (error, "failed to read legacy PD header");
-		return FALSE;
-	}
-
-	/* new location */
-	if (GUINT16_FROM_LE (hdr.vid) != 0x2109) {
-		g_debug ("PD VID was 0x%04x trying new location",
-			 GUINT16_FROM_LE (hdr.vid));
-		if (!fu_vli_device_spi_read_block (FU_VLI_DEVICE (self),
-						   VLI_USBHUB_FLASHMAP_ADDR_PD +
-						   VLI_USBHUB_PD_FLASHMAP_ADDR,
-						   (guint8 *) &hdr, sizeof(hdr), error)) {
-			g_prefix_error (error, "failed to read PD header");
-			return FALSE;
-		}
-	}
-
-	/* just empty space */
-	if (hdr.fwver == G_MAXUINT32) {
-		g_debug ("no PD device header found");
-		return TRUE;
-	}
-
 	/* add child */
-	dev = fu_vli_usbhub_pd_device_new (&hdr);
-	fu_device_set_quirks (dev, fu_device_get_quirks (FU_DEVICE (self)));
-	if (!fu_device_probe (dev, &error_local)) {
-		g_warning ("cannot create PD device: %s", error_local->message);
+	dev = fu_vli_usbhub_pd_device_new (self);
+	if (!fu_device_probe (dev, error))
+		return FALSE;
+	if (!fu_device_setup (dev, &error_local)) {
+		if (g_error_matches (error_local,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_FOUND)) {
+			g_debug ("%s", error_local->message);
+		} else {
+			g_warning ("cannot create PD device: %s",
+				   error_local->message);
+		}
 		return TRUE;
 	}
 	fu_device_add_child (FU_DEVICE (self), dev);
@@ -555,7 +536,7 @@ fu_vli_usbhub_device_msp430_setup (FuVliUsbhubDevice *self, GError **error)
 				     FWUPD_ERROR_NOT_FOUND)) {
 			g_debug ("%s", error_local->message);
 		} else {
-			g_warning ("cannot create I²C device: %s",
+			g_warning ("cannot create MSP430 I²C device: %s",
 				   error_local->message);
 		}
 		return TRUE;
@@ -580,7 +561,7 @@ fu_vli_usbhub_device_rtd21xx_setup (FuVliUsbhubDevice *self, GError **error)
 				     FWUPD_ERROR_NOT_FOUND)) {
 			g_debug ("%s", error_local->message);
 		} else {
-			g_warning ("cannot create I²C device: %s",
+			g_warning ("cannot create RTD21XX I²C device: %s",
 				   error_local->message);
 		}
 		return TRUE;
@@ -619,7 +600,7 @@ fu_vli_usbhub_device_setup (FuVliDevice *device, GError **error)
 	if (!fu_vli_device_spi_read_block (FU_VLI_DEVICE (self), VLI_USBHUB_FLASHMAP_ADDR_HD1,
 					   (guint8 *) &self->hd1_hdr,
 					   sizeof(self->hd1_hdr), error)) {
-		g_prefix_error (error, "failed to read HD1 header");
+		g_prefix_error (error, "failed to read HD1 header: ");
 		return FALSE;
 	}
 
@@ -656,7 +637,7 @@ fu_vli_usbhub_device_setup (FuVliDevice *device, GError **error)
 		if (!fu_vli_device_spi_read_block (FU_VLI_DEVICE (self), VLI_USBHUB_FLASHMAP_ADDR_HD2,
 						   (guint8 *) &self->hd2_hdr,
 						   sizeof(self->hd2_hdr), error)) {
-			g_prefix_error (error, "failed to read HD2 header");
+			g_prefix_error (error, "failed to read HD2 header: ");
 			return FALSE;
 		}
 	}
@@ -693,28 +674,7 @@ fu_vli_usbhub_device_prepare_firmware (FuDevice *device,
 	guint16 device_id;
 	g_autoptr(FuFirmware) firmware = fu_vli_usbhub_firmware_new ();
 
-	/* check size */
-	if (g_bytes_get_size (fw) < fu_device_get_firmware_size_min (device)) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_FILE,
-			     "firmware too small, got 0x%x, expected >= 0x%x",
-			     (guint) g_bytes_get_size (fw),
-			     (guint) fu_device_get_firmware_size_min (device));
-		return NULL;
-	}
-	if (g_bytes_get_size (fw) > fu_device_get_firmware_size_max (device)) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_FILE,
-			     "firmware too large, got 0x%x, expected <= 0x%x",
-			     (guint) g_bytes_get_size (fw),
-			     (guint) fu_device_get_firmware_size_max (device));
-		return NULL;
-	}
-
 	/* check is compatible with firmware */
-	fu_device_set_status (device, FWUPD_STATUS_DECOMPRESSING);
 	if (!fu_firmware_parse (firmware, fw, flags, error))
 		return NULL;
 	device_kind = fu_vli_usbhub_firmware_get_device_kind (FU_VLI_USBHUB_FIRMWARE (firmware));
@@ -784,7 +744,7 @@ fu_vli_usbhub_device_update_v2_recovery (FuVliUsbhubDevice *self, GBytes *fw, GE
 	fu_device_set_status (FU_DEVICE (self), FWUPD_STATUS_DEVICE_ERASE);
 	for (guint32 addr = 0; addr < bufsz; addr += 0x1000) {
 		if (!fu_vli_device_spi_erase_sector (FU_VLI_DEVICE (self), addr, error)) {
-			g_prefix_error (error, "failed to erase sector @0x%x", addr);
+			g_prefix_error (error, "failed to erase sector @0x%x: ", addr);
 			return FALSE;
 		}
 		fu_device_set_progress_full (FU_DEVICE (self), (gsize) addr, bufsz);
@@ -888,7 +848,7 @@ fu_vli_usbhub_device_update_v2 (FuVliUsbhubDevice *self, FuFirmware *firmware, G
 						   (guint8 *) &self->hd1_hdr, sizeof(hdr),
 						   error)) {
 			g_prefix_error (error,
-					"failed to read root header from 0x%x",
+					"failed to read root header from 0x%x: ",
 					(guint) VLI_USBHUB_FLASHMAP_ADDR_HD1_BACKUP);
 			return FALSE;
 		}
@@ -969,18 +929,14 @@ fu_vli_usbhub_device_update_v2 (FuVliUsbhubDevice *self, FuFirmware *firmware, G
 	return TRUE;
 }
 
-static FuFirmware *
-fu_vli_usbhub_device_read_firmware (FuDevice *device, GError **error)
+static GBytes *
+fu_vli_usbhub_device_dump_firmware (FuDevice *device, GError **error)
 {
 	FuVliUsbhubDevice *self = FU_VLI_USBHUB_DEVICE (device);
-	g_autoptr(GBytes) fw = NULL;
-	fu_device_set_status (FU_DEVICE (self), FWUPD_STATUS_DEVICE_VERIFY);
-	fw = fu_vli_device_spi_read (FU_VLI_DEVICE (self), 0x0,
-				     fu_device_get_firmware_size_max (device),
-				     error);
-	if (fw == NULL)
-		return NULL;
-	return fu_firmware_new_from_bytes (fw);
+	fu_device_set_status (FU_DEVICE (self), FWUPD_STATUS_DEVICE_READ);
+	return fu_vli_device_spi_read (FU_VLI_DEVICE (self), 0x0,
+				       fu_device_get_firmware_size_max (device),
+				       error);
 }
 
 static gboolean
@@ -1028,7 +984,7 @@ fu_vli_usbhub_device_class_init (FuVliUsbhubDeviceClass *klass)
 	FuDeviceClass *klass_device = FU_DEVICE_CLASS (klass);
 	FuVliDeviceClass *klass_vli_device = FU_VLI_DEVICE_CLASS (klass);
 	klass_device->probe = fu_vli_usbhub_device_probe;
-	klass_device->read_firmware = fu_vli_usbhub_device_read_firmware;
+	klass_device->dump_firmware = fu_vli_usbhub_device_dump_firmware;
 	klass_device->write_firmware = fu_vli_usbhub_device_write_firmware;
 	klass_device->prepare_firmware = fu_vli_usbhub_device_prepare_firmware;
 	klass_device->attach = fu_vli_usbhub_device_attach;
