@@ -149,12 +149,11 @@ fu_vli_usbhub_msp430_device_detach (FuDevice *device, GError **error)
 
 	/* avoid power instability by waiting T1 */
 	fu_device_set_status (device, FWUPD_STATUS_DEVICE_RESTART);
-	fu_device_set_progress (device, 0);
-	g_usleep (G_USEC_PER_SEC);
+	fu_device_sleep_with_progress (device, 1); /* seconds */
 
 	/* check the device came back */
 	if (!fu_vli_usbhub_device_i2c_read_status (parent, &status, error)) {
-		g_prefix_error (error, "device did not come back after detach");
+		g_prefix_error (error, "device did not come back after detach: ");
 		return FALSE;
 	}
 	return fu_vli_usbhub_i2c_check_status (status, error);
@@ -167,7 +166,6 @@ fu_vli_usbhub_msp430_device_prepare_firmware (FuDevice *device,
 					      GError **error)
 {
 	g_autoptr(FuFirmware) firmware = fu_ihex_firmware_new ();
-	fu_device_set_status (device, FWUPD_STATUS_DECOMPRESSING);
 	if (!fu_firmware_tokenize (firmware, fw, flags, error))
 		return NULL;
 	return g_steal_pointer (&firmware);
@@ -246,28 +244,8 @@ fu_vli_usbhub_msp430_device_write_firmware (FuDevice *device,
 		FuVliUsbhubDeviceRequest req = { 0x0 };
 		const gchar *line = rcd->buf->str;
 
-		/* check there's enough data for the smallest possible record */
-		if (rcd->buf->len < 11) {
-			g_set_error (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INVALID_FILE,
-				     "line %u is incomplete, length %u",
-				     rcd->ln, (guint) rcd->buf->len);
-			return FALSE;
-		}
-
-		/* check starting token */
-		if (line[0] != ':') {
-			g_set_error (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INVALID_FILE,
-				     "invalid starting token on line %u: %s",
-				     rcd->ln, line);
-			return FALSE;
-		}
-
 		/* length, 16-bit address, type */
-		req.len = fu_firmware_strparse_uint8 (line + 1);
+		req.len = rcd->byte_cnt;
 		if (req.len >= sizeof(req.buf) - 7) {
 			g_set_error (error,
 				     FWUPD_ERROR,
@@ -308,10 +286,7 @@ fu_vli_usbhub_msp430_device_write_firmware (FuDevice *device,
 	/* the device automatically reboots */
 	fu_device_set_status (device, FWUPD_STATUS_DEVICE_RESTART);
 	fu_device_set_progress (device, 0);
-
-	/* as soon as the parent comes back we can query the child */
-	root = fu_device_get_root (device);
-	fu_device_add_flag (root, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
+	fu_device_add_flag (device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
 
 	/* success */
 	return TRUE;
