@@ -19,7 +19,6 @@
 struct _FuVliUsbhubPdDevice
 {
 	FuDevice		 parent_instance;
-	FuVliPdHdr		 hdr;
 	FuVliDeviceKind		 device_kind;
 };
 
@@ -38,10 +37,11 @@ fu_vli_usbhub_pd_device_to_string (FuDevice *device, guint idt, GString *str)
 }
 
 static gboolean
-fu_vli_usbhub_pd_device_probe (FuDevice *device, GError **error)
+fu_vli_usbhub_pd_device_setup (FuDevice *device, GError **error)
 {
+	FuVliPdHdr hdr = { 0x0 };
 	FuVliUsbhubPdDevice *self = FU_VLI_USBHUB_PD_DEVICE (device);
-
+	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE (fu_device_get_parent (device));
 	guint32 fwver;
 	g_autofree gchar *fwver_str = NULL;
 	g_autofree gchar *instance_id0 = NULL;
@@ -49,8 +49,39 @@ fu_vli_usbhub_pd_device_probe (FuDevice *device, GError **error)
 	g_autofree gchar *instance_id2 = NULL;
 	g_autofree gchar *instance_id3 = NULL;
 
+	/* legacy location */
+	if (!fu_vli_device_spi_read_block (FU_VLI_DEVICE (parent),
+					   VLI_USBHUB_FLASHMAP_ADDR_PD_LEGACY +
+					   VLI_USBHUB_PD_FLASHMAP_ADDR_LEGACY,
+					   (guint8 *) &hdr, sizeof(hdr), error)) {
+		g_prefix_error (error, "failed to read legacy PD header: ");
+		return FALSE;
+	}
+
+	/* new location */
+	if (GUINT16_FROM_LE (hdr.vid) != 0x2109) {
+		g_debug ("PD VID was 0x%04x trying new location",
+			 GUINT16_FROM_LE (hdr.vid));
+		if (!fu_vli_device_spi_read_block (FU_VLI_DEVICE (parent),
+						   VLI_USBHUB_FLASHMAP_ADDR_PD +
+						   VLI_USBHUB_PD_FLASHMAP_ADDR,
+						   (guint8 *) &hdr, sizeof(hdr), error)) {
+			g_prefix_error (error, "failed to read PD header: ");
+			return FALSE;
+		}
+	}
+
+	/* just empty space */
+	if (hdr.fwver == G_MAXUINT32) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_FOUND,
+			     "no PD device header found");
+		return FALSE;
+	}
+
 	/* get version */
-	fwver = GUINT32_FROM_BE (self->hdr.fwver);
+	fwver = GUINT32_FROM_BE (hdr.fwver);
 	self->device_kind = fu_vli_pd_common_guess_device_kind (fwver);
 	if (self->device_kind == FU_VLI_DEVICE_KIND_UNKNOWN) {
 		g_set_error (error,
@@ -66,23 +97,23 @@ fu_vli_usbhub_pd_device_probe (FuDevice *device, GError **error)
 	fwver_str = fu_common_version_from_uint32 (fwver, FWUPD_VERSION_FORMAT_QUAD);
 	fu_device_set_version (device, fwver_str);
 	instance_id0 = g_strdup_printf ("USB\\VID_%04X&PID_%04X&APP_%02X",
-					GUINT16_FROM_LE (self->hdr.vid),
-					GUINT16_FROM_LE (self->hdr.pid),
+					GUINT16_FROM_LE (hdr.vid),
+					GUINT16_FROM_LE (hdr.pid),
 					fwver & 0xff);
 	fu_device_add_instance_id (device, instance_id0);
 	instance_id1 = g_strdup_printf ("USB\\VID_%04X&PID_%04X&DEV_%s",
-					GUINT16_FROM_LE (self->hdr.vid),
-					GUINT16_FROM_LE (self->hdr.pid),
+					GUINT16_FROM_LE (hdr.vid),
+					GUINT16_FROM_LE (hdr.pid),
 					fu_vli_common_device_kind_to_string (self->device_kind));
 	fu_device_add_instance_id (device, instance_id1);
 
 	/* add standard GUIDs in order of priority */
 	instance_id2 = g_strdup_printf ("USB\\VID_%04X&PID_%04X",
-					GUINT16_FROM_LE (self->hdr.vid),
-					GUINT16_FROM_LE (self->hdr.pid));
+					GUINT16_FROM_LE (hdr.vid),
+					GUINT16_FROM_LE (hdr.pid));
 	fu_device_add_instance_id (device, instance_id2);
 	instance_id3 = g_strdup_printf ("USB\\VID_%04X",
-					GUINT16_FROM_LE (self->hdr.vid));
+					GUINT16_FROM_LE (hdr.vid));
 	fu_device_add_instance_id_full (device, instance_id3,
 					FU_DEVICE_INSTANCE_FLAG_ONLY_QUIRKS);
 
@@ -94,6 +125,19 @@ fu_vli_usbhub_pd_device_probe (FuDevice *device, GError **error)
 	return TRUE;
 }
 
+static gboolean
+fu_vli_usbhub_pd_device_reload (FuDevice *device, GError **error)
+{
+	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE (fu_device_get_parent (device));
+	g_autoptr(FuDeviceLocker) locker = NULL;
+
+	/* open parent device */
+	locker = fu_device_locker_new (parent, error);
+	if (locker == NULL)
+		return FALSE;
+	return fu_vli_usbhub_pd_device_setup (device, error);
+}
+
 static FuFirmware *
 fu_vli_usbhub_pd_device_prepare_firmware (FuDevice *device,
 				       GBytes *fw,
@@ -104,28 +148,7 @@ fu_vli_usbhub_pd_device_prepare_firmware (FuDevice *device,
 	FuVliDeviceKind device_kind;
 	g_autoptr(FuFirmware) firmware = fu_vli_pd_firmware_new ();
 
-	/* check size */
-	if (g_bytes_get_size (fw) < fu_device_get_firmware_size_min (device)) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_FILE,
-			     "firmware too small, got 0x%x, expected >= 0x%x",
-			     (guint) g_bytes_get_size (fw),
-			     (guint) fu_device_get_firmware_size_min (device));
-		return NULL;
-	}
-	if (g_bytes_get_size (fw) > fu_device_get_firmware_size_max (device)) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_FILE,
-			     "firmware too large, got 0x%x, expected <= 0x%x",
-			     (guint) g_bytes_get_size (fw),
-			     (guint) fu_device_get_firmware_size_max (device));
-		return NULL;
-	}
-
 	/* check is compatible with firmware */
-	fu_device_set_status (device, FWUPD_STATUS_DECOMPRESSING);
 	if (!fu_firmware_parse (firmware, fw, flags, error))
 		return NULL;
 	device_kind = fu_vli_pd_firmware_get_kind (FU_VLI_PD_FIRMWARE (firmware));
@@ -144,13 +167,12 @@ fu_vli_usbhub_pd_device_prepare_firmware (FuDevice *device,
 	return g_steal_pointer (&firmware);
 }
 
-static FuFirmware *
-fu_vli_usbhub_pd_device_read_firmware (FuDevice *device, GError **error)
+static GBytes *
+fu_vli_usbhub_pd_device_dump_firmware (FuDevice *device, GError **error)
 {
 	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE (fu_device_get_parent (device));
 	FuVliUsbhubPdDevice *self = FU_VLI_USBHUB_PD_DEVICE (device);
 	g_autoptr(FuDeviceLocker) locker = NULL;
-	g_autoptr(GBytes) fw = NULL;
 
 	/* open device */
 	locker = fu_device_locker_new (parent, error);
@@ -158,14 +180,11 @@ fu_vli_usbhub_pd_device_read_firmware (FuDevice *device, GError **error)
 		return NULL;
 
 	/* read */
-	fu_device_set_status (FU_DEVICE (device), FWUPD_STATUS_DEVICE_VERIFY);
-	fw = fu_vli_device_spi_read (FU_VLI_DEVICE (parent),
-				     fu_vli_common_device_kind_get_offset (self->device_kind),
-				     fu_device_get_firmware_size_max (device),
-				     error);
-	if (fw == NULL)
-		return NULL;
-	return fu_firmware_new_from_bytes (fw);
+	fu_device_set_status (FU_DEVICE (device), FWUPD_STATUS_DEVICE_READ);
+	return fu_vli_device_spi_read (FU_VLI_DEVICE (parent),
+				       fu_vli_common_device_kind_get_offset (self->device_kind),
+				       fu_device_get_firmware_size_max (device),
+				       error);
 }
 
 static gboolean
@@ -228,6 +247,7 @@ fu_vli_usbhub_pd_device_init (FuVliUsbhubPdDevice *self)
 	fu_device_set_protocol (FU_DEVICE (self), "com.vli.usbhub");
 	fu_device_add_flag (FU_DEVICE (self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag (FU_DEVICE (self), FWUPD_DEVICE_FLAG_CAN_VERIFY_IMAGE);
+	fu_device_add_flag (FU_DEVICE (self), FWUPD_DEVICE_FLAG_NO_GUID_MATCHING);
 	fu_device_set_version_format (FU_DEVICE (self), FWUPD_VERSION_FORMAT_QUAD);
 	fu_device_set_install_duration (FU_DEVICE (self), 15); /* seconds */
 	fu_device_set_logical_id (FU_DEVICE (self), "PD");
@@ -239,17 +259,19 @@ fu_vli_usbhub_pd_device_class_init (FuVliUsbhubPdDeviceClass *klass)
 {
 	FuDeviceClass *klass_device = FU_DEVICE_CLASS (klass);
 	klass_device->to_string = fu_vli_usbhub_pd_device_to_string;
-	klass_device->probe = fu_vli_usbhub_pd_device_probe;
+	klass_device->setup = fu_vli_usbhub_pd_device_setup;
+	klass_device->reload = fu_vli_usbhub_pd_device_reload;
 	klass_device->attach = fu_vli_usbhub_pd_device_attach;
-	klass_device->read_firmware = fu_vli_usbhub_pd_device_read_firmware;
+	klass_device->dump_firmware = fu_vli_usbhub_pd_device_dump_firmware;
 	klass_device->write_firmware = fu_vli_usbhub_pd_device_write_firmware;
 	klass_device->prepare_firmware = fu_vli_usbhub_pd_device_prepare_firmware;
 }
 
 FuDevice *
-fu_vli_usbhub_pd_device_new (FuVliPdHdr *hdr)
+fu_vli_usbhub_pd_device_new (FuVliUsbhubDevice *parent)
 {
-	FuVliUsbhubPdDevice *self = g_object_new (FU_TYPE_VLI_USBHUB_PD_DEVICE, NULL);
-	memcpy (&self->hdr, hdr, sizeof(self->hdr));
+	FuVliUsbhubPdDevice *self = g_object_new (FU_TYPE_VLI_USBHUB_PD_DEVICE,
+						  "parent", parent,
+						   NULL);
 	return FU_DEVICE (self);
 }
