@@ -354,7 +354,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware (FuDevice *device,
 		return FALSE;
 	}
 	if (read_buf[0] != ISP_STATUS_IDLE_SUCCESS) {
-		g_prefix_error (error, "failed project ID with error 0x%02x", read_buf[0]);
+		g_prefix_error (error, "failed project ID with error 0x%02x: ", read_buf[0]);
 		return FALSE;
 	}
 
@@ -448,10 +448,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware (FuDevice *device,
 
 	/* the device needs some time to restart with the new firmware before
 	 * it can be queried again */
-	for (guint i = 0; i < 100; i++) {
-		g_usleep (200000);
-		fu_device_set_progress (device, i);
-	}
+	fu_device_sleep_with_progress (device, 20);
 
 	/* success */
 	return TRUE;
