@@ -38,8 +38,6 @@ struct _FuDeviceList
 	GObject			 parent_instance;
 	GPtrArray		*devices;	/* of FuDeviceItem */
 	GRWLock			 devices_mutex;
-	GMainLoop		*replug_loop;	/* block waiting for replug */
-	guint			 replug_id;	/* timeout the loop */
 };
 
 enum {
@@ -81,6 +79,60 @@ fu_device_list_emit_device_changed (FuDeviceList *self, FuDevice *device)
 	g_signal_emit (self, signals[SIGNAL_CHANGED], 0, device);
 }
 
+/* we cannot use fu_device_get_children() as this will not find "parent-only"
+ * logical relationships added using fu_device_add_parent_guid() */
+static GPtrArray *
+fu_device_list_get_children (FuDeviceList *self, FuDevice *device)
+{
+	GPtrArray *devices = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	g_rw_lock_reader_lock (&self->devices_mutex);
+	for (guint i = 0; i < self->devices->len; i++) {
+		FuDeviceItem *item = g_ptr_array_index (self->devices, i);
+		if (device == fu_device_get_parent (item->device))
+			g_ptr_array_add (devices, g_object_ref (item->device));
+	}
+	g_rw_lock_reader_unlock (&self->devices_mutex);
+	return devices;
+}
+
+static void
+fu_device_list_depsolve_order_full (FuDeviceList *self, FuDevice *device, guint depth)
+{
+	g_autoptr(GPtrArray) children = NULL;
+
+	/* ourself */
+	fu_device_set_order (device, depth);
+
+	/* optional children */
+	children = fu_device_list_get_children (self, device);
+	for (guint i = 0; i < children->len; i++) {
+		FuDevice *child = g_ptr_array_index (children, i);
+		if (fu_device_has_flag (child, FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST)) {
+			fu_device_list_depsolve_order_full (self, child, depth + 1);
+		} else {
+			fu_device_list_depsolve_order_full (self, child, depth - 1);
+		}
+	}
+}
+
+/**
+ * fu_device_list_depsolve_order:
+ * @self: A #FuDeviceList
+ * @device: A #FuDevice
+ *
+ * Sets the device order using the logical parent->child relationships -- by default
+ * the child is updated first, unless the device has set flag
+ * %FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST.
+ *
+ * Since: 1.5.0
+ **/
+void
+fu_device_list_depsolve_order (FuDeviceList *self, FuDevice *device)
+{
+	g_autoptr(FuDevice) root = fu_device_get_root (device);
+	fu_device_list_depsolve_order_full (self, root, 0);
+}
+
 /**
  * fu_device_list_get_all:
  * @self: A #FuDeviceList
@@ -588,11 +640,9 @@ fu_device_list_replace (FuDeviceList *self, FuDeviceItem *item, FuDevice *device
 	fu_device_list_emit_device_changed (self, device);
 
 	/* we were waiting for this... */
-	if (fu_device_has_flag (item->device_old, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG) &&
-	    g_main_loop_is_running (self->replug_loop)) {
-		g_debug ("quitting replug loop");
+	if (fu_device_has_flag (item->device_old, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG)) {
+		g_debug ("device came back, clearing flag");
 		fu_device_remove_flag (item->device_old, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
-		g_main_loop_quit (self->replug_loop);
 	}
 }
 
@@ -628,20 +678,13 @@ fu_device_list_add (FuDeviceList *self, FuDevice *device)
 
 	/* is the device waiting to be replugged? */
 	item = fu_device_list_find_by_id (self, fu_device_get_id (device), NULL);
-	if (item != NULL && item->remove_id != 0) {
+	if (item != NULL) {
 		g_debug ("found existing device %s, reusing item",
 			 fu_device_get_id (item->device));
 		fu_device_list_replace (self, item, device);
 		return;
 	}
 
-	/* verify the device does not already exist */
-	if (item != NULL) {
-		g_debug ("device %s already exists, ignoring",
-			 fu_device_get_id (item->device));
-		return;
-	}
-
 	/* verify a device with same connection does not already exist */
 	item = fu_device_list_find_by_connection (self,
 						  fu_device_get_physical_id (device),
@@ -681,26 +724,32 @@ fu_device_list_add (FuDeviceList *self, FuDevice *device)
 		       fu_device_get_protocol (device)) == 0) {
 		if (!fu_device_has_flag (device, FWUPD_DEVICE_FLAG_NO_GUID_MATCHING)) {
 			if (fu_device_get_priority (device) < fu_device_get_priority (item->device)) {
-				g_debug ("ignoring device %s [%s] as better device %s [%s] already exists",
+				g_debug ("ignoring device %s [%s:%s] as better device %s [%s:%s] already exists",
 					 fu_device_get_id (device),
 					 fu_device_get_plugin (device),
+					 g_type_name (fu_device_get_specialized_gtype (device)),
 					 fu_device_get_id (item->device),
-					 fu_device_get_plugin (item->device));
+					 fu_device_get_plugin (item->device),
+					 g_type_name (fu_device_get_specialized_gtype (item->device)));
 				return;
 			}
 			if (fu_device_get_priority (device) == fu_device_get_priority (item->device)) {
-				g_warning ("ignoring device %s [%s] existing device %s [%s] already exists",
+				g_warning ("ignoring device %s [%s:%s] existing device %s [%s:%s] already exists",
 					   fu_device_get_id (device),
 					   fu_device_get_plugin (device),
+					   g_type_name (fu_device_get_specialized_gtype (device)),
 					   fu_device_get_id (item->device),
-					   fu_device_get_plugin (item->device));
+					   fu_device_get_plugin (item->device),
+					   g_type_name (fu_device_get_specialized_gtype (item->device)));
 				return;
 			}
-			g_debug ("removing device %s [%s] as better device %s [%s] added",
+			g_debug ("removing device %s [%s:%s] as better device %s [%s:%s] added",
 				 fu_device_get_id (item->device),
 				 fu_device_get_plugin (item->device),
+				 g_type_name (fu_device_get_specialized_gtype (item->device)),
 				 fu_device_get_id (device),
-				 fu_device_get_plugin (device));
+				 fu_device_get_plugin (device),
+				 g_type_name (fu_device_get_specialized_gtype (device)));
 			fu_device_list_remove (self, item->device);
 		} else {
 			g_debug ("not adding matching %s for device add, use "
@@ -749,18 +798,19 @@ fu_device_list_get_by_guid (FuDeviceList *self, const gchar *guid, GError **erro
 	return NULL;
 }
 
-static gboolean
-fu_device_list_replug_cb (gpointer user_data)
+/* count devices that are disconnected and are waiting to be replugged */
+static guint
+fu_device_list_devices_wait_removed (FuDeviceList *self)
 {
-	FuDeviceList *self = FU_DEVICE_LIST (user_data);
-
-	/* no longer valid */
-	self->replug_id = 0;
-
-	/* quit loop */
-	g_debug ("device did not replug");
-	g_main_loop_quit (self->replug_loop);
-	return FALSE;
+	guint cnt = 0;
+	g_rw_lock_reader_lock (&self->devices_mutex);
+	for (guint i = 0; i < self->devices->len; i++) {
+		FuDeviceItem *item = g_ptr_array_index (self->devices, i);
+		if (item->remove_id != 0)
+			cnt++;
+	}
+	g_rw_lock_reader_unlock (&self->devices_mutex);
+	return cnt;
 }
 
 /**
@@ -783,11 +833,13 @@ fu_device_list_wait_for_replug (FuDeviceList *self, FuDevice *device, GError **e
 {
 	FuDeviceItem *item;
 	guint remove_delay;
+	guint wait_removed;
+	guint wait_removed_old = 0;
+	g_autoptr(GTimer) timer = g_timer_new ();
 
 	g_return_val_if_fail (FU_IS_DEVICE_LIST (self), FALSE);
 	g_return_val_if_fail (FU_IS_DEVICE (device), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-	g_return_val_if_fail (self->replug_id == 0, FALSE);
 
 	/* not found */
 	item = fu_device_list_find_by_device (self, device);
@@ -826,14 +878,20 @@ fu_device_list_wait_for_replug (FuDeviceList *self, FuDevice *device, GError **e
 	}
 
 	/* time to unplug and then re-plug */
-	self->replug_id = g_timeout_add (remove_delay, fu_device_list_replug_cb, self);
-	g_main_loop_run (self->replug_loop);
-
-	/* cancel timeout if still pending */
-	if (self->replug_id != 0) {
-		g_source_remove (self->replug_id);
-		self->replug_id = 0;
-	}
+	do {
+		/* count how many devices are in the remove waiting state */
+		wait_removed = fu_device_list_devices_wait_removed (self);
+		if (wait_removed != wait_removed_old) {
+			g_debug ("devices in wait_removed: %u -> %u",
+				 wait_removed_old, wait_removed);
+			wait_removed_old = wait_removed;
+		}
+		g_usleep (1000);
+		g_main_context_iteration (NULL, FALSE);
+		if (!fu_device_has_flag (item->device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG) &&
+		    wait_removed == 0)
+			break;
+	} while (g_timer_elapsed (timer, NULL) * 1000.f < remove_delay);
 
 	/* device was not added back to the device list */
 	if (fu_device_has_flag (item->device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG)) {
@@ -947,7 +1005,6 @@ static void
 fu_device_list_init (FuDeviceList *self)
 {
 	self->devices = g_ptr_array_new_with_free_func ((GDestroyNotify) fu_device_list_item_free);
-	self->replug_loop = g_main_loop_new (NULL, FALSE);
 	g_rw_lock_init (&self->devices_mutex);
 }
 
@@ -956,11 +1013,8 @@ fu_device_list_finalize (GObject *obj)
 {
 	FuDeviceList *self = FU_DEVICE_LIST (obj);
 
-	if (self->replug_id != 0)
-		g_source_remove (self->replug_id);
-	g_ptr_array_unref (self->devices);
-	g_main_loop_unref (self->replug_loop);
 	g_rw_lock_clear (&self->devices_mutex);
+	g_ptr_array_unref (self->devices);
 
 	G_OBJECT_CLASS (fu_device_list_parent_class)->finalize (obj);
 }
