@@ -14,12 +14,19 @@
 #include <glib/gi18n.h>
 #include <glib-unix.h>
 #include <locale.h>
+#ifdef HAVE_POLKIT
 #include <polkit/polkit.h>
+#endif
+#ifdef HAVE_SYSTEMD
+#include <systemd/sd-daemon.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <jcat.h>
 
 #include "fwupd-device-private.h"
+#include "fwupd-plugin-private.h"
+#include "fwupd-security-attr-private.h"
 #include "fwupd-release-private.h"
 #include "fwupd-remote-private.h"
 #include "fwupd-resources.h"
@@ -29,14 +36,23 @@
 #include "fu-device-private.h"
 #include "fu-engine.h"
 #include "fu-install-task.h"
+#include "fu-security-attrs-private.h"
 
+#ifdef HAVE_POLKIT
 #ifndef HAVE_POLKIT_0_114
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunused-function"
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitAuthorizationResult, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitSubject, g_object_unref)
 #pragma clang diagnostic pop
-#endif
+#endif /* HAVE_POLKIT_0_114 */
+#endif /* HAVE_POLKIT */
+
+typedef enum {
+	FU_MAIN_MACHINE_KIND_PHYSICAL,
+	FU_MAIN_MACHINE_KIND_VIRTUAL,
+	FU_MAIN_MACHINE_KIND_CONTAINER,
+} FuMainMachineKind;
 
 typedef struct {
 	GDBusConnection		*connection;
@@ -48,11 +64,14 @@ typedef struct {
 #if GLIB_CHECK_VERSION(2,63,3)
 	GMemoryMonitor		*memory_monitor;
 #endif
+#ifdef HAVE_POLKIT
 	PolkitAuthority		*authority;
+#endif
 	guint			 owner_id;
 	FuEngine		*engine;
 	gboolean		 update_in_progress;
 	gboolean		 pending_sigterm;
+	FuMainMachineKind	 machine_kind;
 } FuMainPrivate;
 
 static gboolean
@@ -260,6 +279,22 @@ fu_main_device_array_to_variant (FuMainPrivate *priv, FuEngineRequest *request,
 	return g_variant_new ("(aa{sv})", &builder);
 }
 
+static GVariant *
+fu_main_plugin_array_to_variant (GPtrArray *plugins)
+{
+	GVariantBuilder builder;
+
+	g_return_val_if_fail (plugins->len > 0, NULL);
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+
+	for (guint i = 0; i < plugins->len; i++) {
+		FuDevice *plugin = g_ptr_array_index (plugins, i);
+		GVariant *tmp = fwupd_plugin_to_variant (FWUPD_PLUGIN (plugin));
+		g_variant_builder_add_value (&builder, tmp);
+	}
+	return g_variant_new ("(aa{sv})", &builder);
+}
+
 static GVariant *
 fu_main_release_array_to_variant (GPtrArray *results)
 {
@@ -305,7 +340,9 @@ fu_main_result_array_to_variant (GPtrArray *results)
 typedef struct {
 	GDBusMethodInvocation	*invocation;
 	FuEngineRequest		*request;
+#ifdef HAVE_POLKIT
 	PolkitSubject		*subject;
+#endif
 	GPtrArray		*install_tasks;
 	GPtrArray		*action_ids;
 	GPtrArray		*checksums;
@@ -324,8 +361,10 @@ fu_main_auth_helper_free (FuMainAuthHelper *helper)
 {
 	if (helper->blob_cab != NULL)
 		g_bytes_unref (helper->blob_cab);
+#ifdef HAVE_POLKIT
 	if (helper->subject != NULL)
 		g_object_unref (helper->subject);
+#endif
 	if (helper->silo != NULL)
 		g_object_unref (helper->silo);
 	if (helper->request != NULL)
@@ -349,6 +388,7 @@ fu_main_auth_helper_free (FuMainAuthHelper *helper)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(FuMainAuthHelper, fu_main_auth_helper_free)
 #pragma clang diagnostic pop
 
+#ifdef HAVE_POLKIT
 /* error may or may not already have been set */
 static gboolean
 fu_main_authorization_is_valid (PolkitAuthorizationResult *auth, GError **error)
@@ -376,15 +416,30 @@ fu_main_authorization_is_valid (PolkitAuthorizationResult *auth, GError **error)
 	/* success */
 	return TRUE;
 }
+#else
+static gboolean
+fu_main_authorization_is_trusted (FuEngineRequest *request, GError **error)
+{
+	FwupdDeviceFlags flags = fu_engine_request_get_device_flags (request);
+	if ((flags & FWUPD_DEVICE_FLAG_TRUSTED) == 0) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_AUTH_FAILED,
+				     "permission denied: untrusted client process");
+		return FALSE;
+	}
+	return TRUE;
+}
+#endif /* HAVE_POLKIT */
 
 static void
 fu_main_authorize_unlock_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
-
 	/* get result */
 	fu_main_set_status (helper->priv, FWUPD_STATUS_IDLE);
 	auth = polkit_authority_check_authorization_finish (POLKIT_AUTHORITY (source),
@@ -393,6 +448,12 @@ fu_main_authorize_unlock_cb (GObject *source, GAsyncResult *res, gpointer user_d
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	/* authenticated */
 	if (!fu_engine_unlock (helper->priv->engine, helper->device_id, &error)) {
@@ -409,6 +470,7 @@ fu_main_authorize_set_approved_firmware_cb (GObject *source, GAsyncResult *res,
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -419,6 +481,12 @@ fu_main_authorize_set_approved_firmware_cb (GObject *source, GAsyncResult *res,
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	/* success */
 	for (guint i = 0; i < helper->checksums->len; i++) {
@@ -433,6 +501,7 @@ fu_main_authorize_set_blocked_firmware_cb (GObject *source, GAsyncResult *res, g
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -443,6 +512,12 @@ fu_main_authorize_set_blocked_firmware_cb (GObject *source, GAsyncResult *res, g
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	/* success */
 	if (!fu_engine_set_blocked_firmware (helper->priv->engine, helper->checksums, &error)) {
@@ -458,6 +533,7 @@ fu_main_authorize_self_sign_cb (GObject *source, GAsyncResult *res, gpointer use
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autofree gchar *sig = NULL;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -468,6 +544,12 @@ fu_main_authorize_self_sign_cb (GObject *source, GAsyncResult *res, gpointer use
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	/* authenticated */
 	sig = fu_engine_self_sign (helper->priv->engine, helper->value, helper->flags, &error);
@@ -485,6 +567,7 @@ fu_main_modify_config_cb (GObject *source, GAsyncResult *res, gpointer user_data
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -494,6 +577,12 @@ fu_main_modify_config_cb (GObject *source, GAsyncResult *res, gpointer user_data
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	if (!fu_engine_modify_config (helper->priv->engine, helper->key, helper->value, &error)) {
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
@@ -509,6 +598,7 @@ fu_main_authorize_activate_cb (GObject *source, GAsyncResult *res, gpointer user
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -519,6 +609,7 @@ fu_main_authorize_activate_cb (GObject *source, GAsyncResult *res, gpointer user
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#endif /* HAVE_POLKIT */
 
 	/* authenticated */
 	if (!fu_engine_activate (helper->priv->engine, helper->device_id, &error)) {
@@ -535,6 +626,7 @@ fu_main_authorize_verify_update_cb (GObject *source, GAsyncResult *res, gpointer
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -545,6 +637,12 @@ fu_main_authorize_verify_update_cb (GObject *source, GAsyncResult *res, gpointer
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	/* authenticated */
 	if (!fu_engine_verify_update (helper->priv->engine, helper->device_id, &error)) {
@@ -561,6 +659,7 @@ fu_main_authorize_modify_remote_cb (GObject *source, GAsyncResult *res, gpointer
 {
 	g_autoptr(FuMainAuthHelper) helper = (FuMainAuthHelper *) user_data;
 	g_autoptr(GError) error = NULL;
+#ifdef HAVE_POLKIT
 	g_autoptr(PolkitAuthorizationResult) auth = NULL;
 
 	/* get result */
@@ -571,6 +670,12 @@ fu_main_authorize_modify_remote_cb (GObject *source, GAsyncResult *res, gpointer
 		g_dbus_method_invocation_return_gerror (helper->invocation, error);
 		return;
 	}
+#else
+	if (!fu_main_authorization_is_trusted (helper->request, &error)) {
+		g_dbus_method_invocation_return_gerror (helper->invocation, error);
+		return;
+	}
+#endif /* HAVE_POLKIT */
 
 	/* authenticated */
 	if (!fu_engine_modify_remote (helper->priv->engine,
@@ -588,6 +693,7 @@ fu_main_authorize_modify_remote_cb (GObject *source, GAsyncResult *res, gpointer
 
 static void fu_main_authorize_install_queue (FuMainAuthHelper *helper);
 
+#ifdef HAVE_POLKIT
 static void
 fu_main_authorize_install_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 {
@@ -607,6 +713,7 @@ fu_main_authorize_install_cb (GObject *source, GAsyncResult *res, gpointer user_
 	/* do the next authentication action ID */
 	fu_main_authorize_install_queue (g_steal_pointer (&helper));
 }
+#endif /* HAVE_POLKIT */
 
 static void
 fu_main_authorize_install_queue (FuMainAuthHelper *helper_ref)
@@ -616,6 +723,7 @@ fu_main_authorize_install_queue (FuMainAuthHelper *helper_ref)
 	g_autoptr(GError) error = NULL;
 	gboolean ret;
 
+#ifdef HAVE_POLKIT
 	/* still more things to to authenticate */
 	if (helper->action_ids->len > 0) {
 		g_autofree gchar *action_id = g_strdup (g_ptr_array_index (helper->action_ids, 0));
@@ -629,6 +737,7 @@ fu_main_authorize_install_queue (FuMainAuthHelper *helper_ref)
 						      g_steal_pointer (&helper));
 		return;
 	}
+#endif /* HAVE_POLKIT */
 
 	/* all authenticated, so install all the things */
 	priv->update_in_progress = TRUE;
@@ -744,7 +853,9 @@ fu_main_install_with_helper (FuMainAuthHelper *helper_ref, GError **error)
 		return FALSE;
 
 	/* for each component in the silo */
-	components = xb_silo_query (helper->silo, "components/component", 0, error);
+	components = xb_silo_query (helper->silo,
+				    "components/component[@type='firmware']",
+				    0, error);
 	if (components == NULL)
 		return FALSE;
 	helper->action_ids = g_ptr_array_new_with_free_func (g_free);
@@ -767,10 +878,14 @@ fu_main_install_with_helper (FuMainAuthHelper *helper_ref, GError **error)
 							   task,
 							   helper->flags | FWUPD_INSTALL_FLAG_FORCE,
 							   &error_local)) {
-				g_debug ("first pass requirement on %s:%s failed: %s",
-					 fu_device_get_id (device),
-					 xb_node_query_text (component, "id", NULL),
-					 error_local->message);
+				if (!g_error_matches (error_local,
+						      FWUPD_ERROR,
+						      FWUPD_ERROR_NOT_FOUND)) {
+					g_debug ("first pass requirement on %s:%s failed: %s",
+						 fu_device_get_id (device),
+						 xb_node_query_text (component, "id", NULL),
+						 error_local->message);
+				}
 				g_ptr_array_add (errors, g_steal_pointer (&error_local));
 				continue;
 			}
@@ -789,6 +904,10 @@ fu_main_install_with_helper (FuMainAuthHelper *helper_ref, GError **error)
 				g_ptr_array_add (errors, g_steal_pointer (&error_local));
 				continue;
 			}
+			if (!fu_engine_check_trust (task, &error_local)) {
+				g_ptr_array_add (errors, g_steal_pointer (&error_local));
+				continue;
+			}
 
 			/* if component should have an update message from CAB */
 			fu_device_incorporate_from_component (device, component);
@@ -869,6 +988,12 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		g_dbus_method_invocation_return_value (invocation, val);
 		return;
 	}
+	if (g_strcmp0 (method_name, "GetPlugins") == 0) {
+		g_debug ("Called %s()", method_name);
+		val = fu_main_plugin_array_to_variant (fu_engine_get_plugins (priv->engine));
+		g_dbus_method_invocation_return_value (invocation, val);
+		return;
+	}
 	if (g_strcmp0 (method_name, "GetReleases") == 0) {
 		const gchar *device_id;
 		g_autoptr(GPtrArray) releases = NULL;
@@ -913,11 +1038,38 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						       g_variant_new_tuple (&val, 1));
 		return;
 	}
+	if (g_strcmp0 (method_name, "GetReportMetadata") == 0) {
+		GHashTableIter iter;
+		GVariantBuilder builder;
+		const gchar *key;
+		const gchar *value;
+		g_autoptr(GHashTable) metadata = NULL;
+
+		metadata = fu_engine_get_report_metadata (priv->engine, &error);
+		if (metadata == NULL) {
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
+		}
+		g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
+		g_hash_table_iter_init (&iter, metadata);
+		while (g_hash_table_iter_next (&iter,
+					       (gpointer *) &key,
+					       (gpointer *) &value)) {
+			g_variant_builder_add_value (&builder,
+						     g_variant_new ("{ss}", key, value));
+		}
+		val = g_variant_builder_end (&builder);
+		g_dbus_method_invocation_return_value (invocation,
+						       g_variant_new_tuple (&val, 1));
+		return;
+	}
 	if (g_strcmp0 (method_name, "SetApprovedFirmware") == 0) {
 		g_autofree gchar *checksums_str = NULL;
 		g_auto(GStrv) checksums = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
+#endif /* HAVE_POLKIT */
 
 		g_variant_get (parameters, "(^as)", &checksums);
 		checksums_str = g_strjoinv (",", checksums);
@@ -932,6 +1084,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->checksums = g_ptr_array_new_with_free_func (g_free);
 		for (guint i = 0; checksums[i] != NULL; i++)
 			g_ptr_array_add (helper->checksums, g_strdup (checksums[i]));
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.set-approved-firmware",
@@ -940,14 +1093,18 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_set_approved_firmware_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_set_approved_firmware_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "SetBlockedFirmware") == 0) {
 		g_autofree gchar *checksums_str = NULL;
 		g_auto(GStrv) checksums = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
-
+#endif
 		g_variant_get (parameters, "(^as)", &checksums);
 		checksums_str = g_strjoinv (",", checksums);
 		g_debug ("Called %s(%s)", method_name, checksums_str);
@@ -961,6 +1118,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->checksums = g_ptr_array_new_with_free_func (g_free);
 		for (guint i = 0; checksums[i] != NULL; i++)
 			g_ptr_array_add (helper->checksums, g_strdup (checksums[i]));
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.set-approved-firmware",
@@ -969,6 +1127,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_set_blocked_firmware_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_set_blocked_firmware_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "SelfSign") == 0) {
@@ -976,7 +1137,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		gchar *prop_key;
 		g_autofree gchar *value = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
+#endif
 		g_autoptr(GVariantIter) iter = NULL;
 
 		g_variant_get (parameters, "(sa{sv})", &value, &iter);
@@ -1001,6 +1164,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->value = g_steal_pointer (&value);
 		helper->request = g_steal_pointer (&request);
 		helper->invocation = g_object_ref (invocation);
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.self-sign",
@@ -1009,6 +1173,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_self_sign_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_self_sign_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "GetDowngrades") == 0) {
@@ -1075,6 +1242,21 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		g_dbus_method_invocation_return_value (invocation, val);
 		return;
 	}
+	if (g_strcmp0 (method_name, "GetHostSecurityAttrs") == 0) {
+		g_autoptr(FuSecurityAttrs) attrs = NULL;
+		g_debug ("Called %s()", method_name);
+		if (priv->machine_kind != FU_MAIN_MACHINE_KIND_PHYSICAL) {
+			g_dbus_method_invocation_return_error_literal (invocation,
+								       FWUPD_ERROR,
+								       FWUPD_ERROR_NOT_SUPPORTED,
+								       "HSI unavailable for hypervisor");
+			return;
+		}
+		attrs = fu_engine_get_host_security_attrs (priv->engine);
+		val = fu_security_attrs_to_variant (attrs);
+		g_dbus_method_invocation_return_value (invocation, val);
+		return;
+	}
 	if (g_strcmp0 (method_name, "ClearResults") == 0) {
 		const gchar *device_id;
 		g_variant_get (parameters, "(&s)", &device_id);
@@ -1165,8 +1347,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 	if (g_strcmp0 (method_name, "Unlock") == 0) {
 		const gchar *device_id = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
-
+#endif /* HAVE_POLKIT */
 		g_variant_get (parameters, "(&s)", &device_id);
 		g_debug ("Called %s(%s)", method_name, device_id);
 		if (!fu_main_device_id_valid (device_id, &error)) {
@@ -1181,6 +1364,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->request = g_steal_pointer (&request);
 		helper->invocation = g_object_ref (invocation);
 		helper->device_id = g_strdup (device_id);
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.device-unlock",
@@ -1189,13 +1373,17 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_unlock_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_unlock_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "Activate") == 0) {
 		const gchar *device_id = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
-
+#endif
 		g_variant_get (parameters, "(&s)", &device_id);
 		g_debug ("Called %s(%s)", method_name, device_id);
 		if (!fu_main_device_id_valid (device_id, &error)) {
@@ -1210,6 +1398,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->request = g_steal_pointer (&request);
 		helper->invocation = g_object_ref (invocation);
 		helper->device_id = g_strdup (device_id);
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.device-activate",
@@ -1218,14 +1407,18 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_activate_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_activate_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "ModifyConfig") == 0) {
 		g_autofree gchar *key = NULL;
 		g_autofree gchar *value = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
-
+#endif
 		g_variant_get (parameters, "(ss)", &key, &value);
 		g_debug ("Called %s(%s=%s)", method_name, key, value);
 
@@ -1236,6 +1429,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->value = g_steal_pointer (&value);
 		helper->request = g_steal_pointer (&request);
 		helper->invocation = g_object_ref (invocation);
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.modify-config",
@@ -1244,6 +1438,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_modify_config_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_modify_config_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "ModifyRemote") == 0) {
@@ -1251,8 +1448,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		const gchar *key = NULL;
 		const gchar *value = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
-
+#endif
 		/* check the id exists */
 		g_variant_get (parameters, "(&s&s&s)", &remote_id, &key, &value);
 		g_debug ("Called %s(%s,%s=%s)", method_name, remote_id, key, value);
@@ -1268,6 +1466,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 
 		/* authenticate */
 		fu_main_set_status (priv, FWUPD_STATUS_WAITING_FOR_AUTH);
+#ifdef HAVE_POLKIT
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
 						      "org.freedesktop.fwupd.modify-remote",
@@ -1276,12 +1475,17 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_modify_remote_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_modify_remote_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "VerifyUpdate") == 0) {
 		const gchar *device_id = NULL;
 		g_autoptr(FuMainAuthHelper) helper = NULL;
+#ifdef HAVE_POLKIT
 		g_autoptr(PolkitSubject) subject = NULL;
+#endif
 
 		/* check the id exists */
 		g_variant_get (parameters, "(&s)", &device_id);
@@ -1299,6 +1503,7 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		helper->priv = priv;
 
 		/* authenticate */
+#ifdef HAVE_POLKIT
 		fu_main_set_status (priv, FWUPD_STATUS_WAITING_FOR_AUTH);
 		subject = polkit_system_bus_name_new (sender);
 		polkit_authority_check_authorization (priv->authority, subject,
@@ -1308,6 +1513,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 						      NULL,
 						      fu_main_authorize_verify_update_cb,
 						      g_steal_pointer (&helper));
+#else
+		fu_main_authorize_verify_update_cb (NULL, NULL, g_steal_pointer (&helper));
+#endif /* HAVE_POLKIT */
 		return;
 	}
 	if (g_strcmp0 (method_name, "Verify") == 0) {
@@ -1326,11 +1534,13 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		return;
 	}
 	if (g_strcmp0 (method_name, "SetFeatureFlags") == 0) {
-		guint64 feature_flags = 0;
-		g_variant_get (parameters, "(t)", &feature_flags);
-		g_debug ("Called %s(%" G_GUINT64_FORMAT ")", method_name, feature_flags);
+		FwupdFeatureFlags feature_flags;
+		guint64 feature_flags_u64 = 0;
+		g_variant_get (parameters, "(t)", &feature_flags_u64);
+		g_debug ("Called %s(%" G_GUINT64_FORMAT ")", method_name, feature_flags_u64);
 
 		/* old flags for the same sender will be automatically destroyed */
+		feature_flags = feature_flags_u64;
 		g_hash_table_insert (priv->sender_features,
 				     g_strdup (sender),
 				     g_memdup (&feature_flags, sizeof(feature_flags)));
@@ -1376,9 +1586,17 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 			if (g_strcmp0 (prop_key, "allow-reinstall") == 0 &&
 			    g_variant_get_boolean (prop_value) == TRUE)
 				helper->flags |= FWUPD_INSTALL_FLAG_ALLOW_REINSTALL;
-			if (g_strcmp0 (prop_key, "force") == 0 &&
+			if (g_strcmp0 (prop_key, "allow-branch-switch") == 0 &&
 			    g_variant_get_boolean (prop_value) == TRUE)
+				helper->flags |= FWUPD_INSTALL_FLAG_ALLOW_BRANCH_SWITCH;
+			if (g_strcmp0 (prop_key, "force") == 0 &&
+			    g_variant_get_boolean (prop_value) == TRUE) {
 				helper->flags |= FWUPD_INSTALL_FLAG_FORCE;
+				helper->flags |= FWUPD_INSTALL_FLAG_IGNORE_POWER;
+			}
+			if (g_strcmp0 (prop_key, "ignore-power") == 0 &&
+			    g_variant_get_boolean (prop_value) == TRUE)
+				helper->flags |= FWUPD_INSTALL_FLAG_IGNORE_POWER;
 			if (g_strcmp0 (prop_key, "no-history") == 0 &&
 			    g_variant_get_boolean (prop_value) == TRUE)
 				helper->flags |= FWUPD_INSTALL_FLAG_NO_HISTORY;
@@ -1414,7 +1632,9 @@ fu_main_daemon_method_call (GDBusConnection *connection, const gchar *sender,
 		}
 
 		/* install all the things in the store */
+#ifdef HAVE_POLKIT
 		helper->subject = polkit_system_bus_name_new (sender);
+#endif /* HAVE_POLKIT */
 		if (!fu_main_install_with_helper (g_steal_pointer (&helper), &error)) {
 			g_dbus_method_invocation_return_gerror (invocation, error);
 			return;
@@ -1494,6 +1714,9 @@ fu_main_daemon_get_property (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (property_name, "HostMachineId") == 0)
 		return g_variant_new_string (fu_engine_get_host_machine_id (priv->engine));
 
+	if (g_strcmp0 (property_name, "HostSecurityId") == 0)
+		return g_variant_new_string (fu_engine_get_host_security_id (priv->engine));
+
 	if (g_strcmp0 (property_name, "Interactive") == 0)
 		return g_variant_new_boolean (isatty (fileno (stdout)) != 0);
 
@@ -1552,7 +1775,7 @@ fu_main_on_name_acquired_cb (GDBusConnection *connection,
 			     const gchar *name,
 			     gpointer user_data)
 {
-	g_debug ("FuMain: acquired name: %s", name);
+	g_debug ("acquired name: %s", name);
 }
 
 static void
@@ -1561,7 +1784,7 @@ fu_main_on_name_lost_cb (GDBusConnection *connection,
 			 gpointer user_data)
 {
 	FuMainPrivate *priv = (FuMainPrivate *) user_data;
-	g_debug ("FuMain: lost name: %s", name);
+	g_warning ("another service has claimed the dbus name %s", name);
 	g_main_loop_quit (priv->loop);
 }
 
@@ -1624,6 +1847,30 @@ fu_main_load_introspection (const gchar *filename, GError **error)
 	return g_dbus_node_info_new_for_xml (g_bytes_get_data (data, NULL), error);
 }
 
+static gboolean
+fu_main_is_hypervisor (void)
+{
+	g_autofree gchar *buf = NULL;
+	gsize bufsz = 0;
+	if (!g_file_get_contents ("/proc/cpuinfo", &buf, &bufsz, NULL))
+		return FALSE;
+	return g_strstr_len (buf, (gssize) bufsz, "hypervisor") != NULL;
+}
+
+static gboolean
+fu_main_is_container (void)
+{
+	g_autofree gchar *buf = NULL;
+	gsize bufsz = 0;
+	if (!g_file_get_contents ("/proc/1/cgroup", &buf, &bufsz, NULL))
+		return FALSE;
+	if (g_strstr_len (buf, (gssize) bufsz, "docker") != NULL)
+		return TRUE;
+	if (g_strstr_len (buf, (gssize) bufsz, "lxc") != NULL)
+		return TRUE;
+	return FALSE;
+}
+
 static void
 fu_main_private_free (FuMainPrivate *priv)
 {
@@ -1638,8 +1885,10 @@ fu_main_private_free (FuMainPrivate *priv)
 		g_object_unref (priv->engine);
 	if (priv->connection != NULL)
 		g_object_unref (priv->connection);
+#ifdef HAVE_POLKIT
 	if (priv->authority != NULL)
 		g_object_unref (priv->authority);
+#endif
 	if (priv->argv0_monitor != NULL) {
 		g_file_monitor_cancel (priv->argv0_monitor);
 		g_object_unref (priv->argv0_monitor);
@@ -1750,12 +1999,21 @@ main (int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
+#ifdef HAVE_POLKIT
 	/* get authority */
 	priv->authority = polkit_authority_get_sync (NULL, &error);
 	if (priv->authority == NULL) {
 		g_printerr ("Failed to load authority: %s\n", error->message);
 		return EXIT_FAILURE;
 	}
+#endif
+
+	/* are we a VM? */
+	if (fu_main_is_hypervisor ()) {
+		priv->machine_kind = FU_MAIN_MACHINE_KIND_VIRTUAL;
+	} else if (fu_main_is_container ()) {
+		priv->machine_kind = FU_MAIN_MACHINE_KIND_CONTAINER;
+	}
 
 	/* own the object */
 	priv->owner_id = g_bus_own_name (G_BUS_TYPE_SYSTEM,
@@ -1774,12 +2032,15 @@ main (int argc, char *argv[])
 	else if (timed_exit)
 		g_timeout_add_seconds (5, fu_main_timed_exit_cb, priv->loop);
 
-	g_debug ("Started with locale %s", g_getenv ("LANG"));
-
 	/* wait */
-	g_message ("Daemon ready for requests");
+	g_message ("Daemon ready for requests (locale %s)", g_getenv ("LANG"));
 	g_main_loop_run (priv->loop);
 
+#ifdef HAVE_SYSTEMD
+	/* notify the service manager */
+	sd_notify (0, "STOPPING=1");
+#endif
+
 	/* success */
 	return EXIT_SUCCESS;
 }
